@@ -201,30 +201,30 @@ metadata_op_free (MetadataOpData *metadata_op)
 
 
 static void
-write_metadata_write_buffer_ready_cb (void     *buffer,
-				      gsize     count,
-				      GError   *error,
-				      gpointer  user_data)
+write_metadata_write_buffer_ready_cb (void     **buffer,
+				      gsize      count,
+				      GError    *error,
+				      gpointer   user_data)
 {
 	MetadataOpData        *metadata_op = user_data;
 	GthFileSourceCatalogs *catalogs = metadata_op->catalogs;
 
-	g_free (buffer);
-
 	metadata_op->ready_callback (G_OBJECT (catalogs), error, metadata_op->user_data);
 	metadata_op_free (metadata_op);
 }
 
 
 static void
-write_metadata_load_buffer_ready_cb (void     *buffer,
-				     gsize     count,
-				     GError   *error,
-				     gpointer  user_data)
+write_metadata_load_buffer_ready_cb (void     **buffer,
+				     gsize      count,
+				     GError    *error,
+				     gpointer   user_data)
 {
 	MetadataOpData        *metadata_op = user_data;
 	GthFileSourceCatalogs *catalogs = metadata_op->catalogs;
 	GFile                 *gio_file;
+	void                  *catalog_buffer;
+	gsize                  catalog_size;
 
 	if (error != NULL) {
 		metadata_op->ready_callback (G_OBJECT (catalogs), error, metadata_op->user_data);
@@ -232,7 +232,7 @@ write_metadata_load_buffer_ready_cb (void     *buffer,
 		return;
 	}
 
-	gth_catalog_load_from_data (metadata_op->catalog, buffer, count, &error);
+	gth_catalog_load_from_data (metadata_op->catalog, *buffer, count, &error);
 
 	if (error != NULL) {
 		metadata_op->ready_callback (G_OBJECT (catalogs), error, metadata_op->user_data);
@@ -245,11 +245,14 @@ write_metadata_load_buffer_ready_cb (void     *buffer,
 				       g_file_info_get_attribute_string (metadata_op->file_data->info, "sort::type"),
 				       g_file_info_get_attribute_boolean (metadata_op->file_data->info, "sort::inverse"));
 
-	buffer = gth_catalog_to_data (metadata_op->catalog, &count);
+	gth_hook_invoke ("gth-catalog-read-metadata", metadata_op->catalog, metadata_op->file_data);
+
+	catalog_buffer = gth_catalog_to_data (metadata_op->catalog, &catalog_size);
 	gio_file = gth_catalog_file_to_gio_file (metadata_op->file_data->file);
 	g_write_file_async (gio_file,
-			    buffer,
-			    count,
+			    catalog_buffer,
+			    catalog_size,
+			    TRUE,
 			    G_PRIORITY_DEFAULT,
 			    gth_file_source_get_cancellable (GTH_FILE_SOURCE (metadata_op->catalogs)),
 			    write_metadata_write_buffer_ready_cb,
@@ -355,6 +358,7 @@ read_metadata_info_ready_cb (GList    *files,
 {
 	ReadMetadataOpData *read_metadata = user_data;
 	GthFileData        *result;
+	GFile              *gio_file;
 
 	if (error != NULL) {
 		read_metadata->callback (G_OBJECT (read_metadata->file_source), error, read_metadata->data);
@@ -364,24 +368,15 @@ read_metadata_info_ready_cb (GList    *files,
 
 	result = files->data;
 	g_file_info_copy_into (result->info, read_metadata->file_data->info);
-
 	update_file_info (read_metadata->file_source, read_metadata->file_data->file, read_metadata->file_data->info);
 
-	if (_g_file_attributes_matches_any (read_metadata->attributes, "sort::*")) {
-		GFile *gio_file;
+	gio_file = gth_catalog_file_to_gio_file (read_metadata->file_data->file);
+	gth_catalog_load_from_file_async (gio_file,
+					  gth_file_source_get_cancellable (read_metadata->file_source),
+					  read_metadata_catalog_ready_cb,
+					  read_metadata);
 
-		gio_file = gth_catalog_file_to_gio_file (read_metadata->file_data->file);
-		gth_catalog_load_from_file_async (gio_file,
-						  gth_file_source_get_cancellable (read_metadata->file_source),
-						  read_metadata_catalog_ready_cb,
-						  read_metadata);
-
-		g_object_unref (gio_file);
-	}
-	else {
-		read_metadata->callback (G_OBJECT (read_metadata->file_source), NULL, read_metadata->data);
-		read_metadata_free (read_metadata);
-	}
+	g_object_unref (gio_file);
 }
 
 
@@ -406,8 +401,7 @@ gth_file_source_catalogs_read_metadata (GthFileSource *file_source,
 	gio_file = gth_catalog_file_to_gio_file (file_data->file);
 	files = g_list_prepend (NULL, gio_file);
 	_g_query_all_metadata_async (files,
-				     FALSE,
-				     TRUE,
+				     GTH_LIST_DEFAULT,
 				     attributes,
 				     gth_file_source_get_cancellable (file_source),
 				     read_metadata_info_ready_cb,
@@ -670,15 +664,12 @@ typedef struct {
 	gpointer          user_data;
 	GList            *files;
 	GthCatalog       *catalog;
-	char             *buffer;
-	gsize             length;
 } CopyOpData;
 
 
 static void
 copy_op_data_free (CopyOpData *cod)
 {
-	g_free (cod->buffer);
 	_g_object_unref (cod->catalog);
 	_g_object_list_unref (cod->files);
 	_g_object_list_unref (cod->file_list);
@@ -689,10 +680,10 @@ copy_op_data_free (CopyOpData *cod)
 
 
 static void
-catalog_save_done_cb (void     *buffer,
-		      gsize     count,
-		      GError   *error,
-		      gpointer  user_data)
+catalog_save_done_cb (void     **buffer,
+		      gsize      count,
+		      GError    *error,
+		      gpointer   user_data)
 {
 	CopyOpData *cod = user_data;
 
@@ -714,6 +705,8 @@ catalog_ready_cb (GObject  *catalog,
 {
 	CopyOpData *cod = user_data;
 	GList      *scan;
+	char       *buffer;
+	gsize       size;
 	GFile      *gio_file;
 
 	if (error != NULL) {
@@ -727,11 +720,12 @@ catalog_ready_cb (GObject  *catalog,
 	for (scan = cod->files; scan; scan = scan->next)
 		gth_catalog_insert_file (cod->catalog, (GFile *) scan->data, -1);
 
-	cod->buffer = gth_catalog_to_data (cod->catalog, &cod->length);
+	buffer = gth_catalog_to_data (cod->catalog, &size);
 	gio_file = gth_catalog_file_to_gio_file (cod->destination->file);
 	g_write_file_async (gio_file,
-			    cod->buffer,
-			    cod->length,
+			    buffer,
+			    size,
+			    TRUE,
 			    G_PRIORITY_DEFAULT,
 			    NULL,
 			    catalog_save_done_cb,
@@ -801,8 +795,7 @@ gth_file_source_catalogs_copy (GthFileSource    *file_source,
 	}
 
 	_g_query_info_async (cod->file_list,
-			     FALSE,
-			     TRUE,
+			     GTH_LIST_DEFAULT,
 			     GFILE_NAME_TYPE_ATTRIBUTES,
 			     gth_file_source_get_cancellable (file_source),
 			     copy__file_list_info_ready_cb,
@@ -842,15 +835,13 @@ reorder_data_free (ReorderData *reorder_data)
 
 
 static void
-reorder_buffer_ready_cb (void     *buffer,
-		         gsize     count,
-		         GError   *error,
-		         gpointer  user_data)
+reorder_buffer_ready_cb (void     **buffer,
+		         gsize      count,
+		         GError    *error,
+		         gpointer   user_data)
 {
 	ReorderData *reorder_data = user_data;
 
-	g_free (buffer);
-
 	gth_monitor_order_changed (gth_main_get_default_monitor (),
 				   reorder_data->destination->file,
 				   reorder_data->new_order);
@@ -904,7 +895,7 @@ reorder_catalog_ready_cb (GObject  *object,
 	ReorderData *reorder_data = user_data;
 	GthCatalog  *catalog;
 	char        *buffer;
-	gsize        buffer_size;
+	gsize        size;
 	GFile       *gio_file;
 
 	if (error != NULL) {
@@ -917,11 +908,12 @@ reorder_catalog_ready_cb (GObject  *object,
 	reorder_data->new_order = reorder_catalog_list (catalog, reorder_data->file_list, reorder_data->dest_pos);
 	gth_catalog_set_order (catalog, "general::unsorted", FALSE);
 
-	buffer = gth_catalog_to_data (catalog, &buffer_size);
+	buffer = gth_catalog_to_data (catalog, &size);
 	gio_file = gth_file_source_to_gio_file (reorder_data->file_source, reorder_data->destination->file);
 	g_write_file_async (gio_file,
 			    buffer,
-			    buffer_size,
+			    size,
+			    TRUE,
 			    G_PRIORITY_DEFAULT,
 			    gth_file_source_get_cancellable (reorder_data->file_source),
 			    reorder_buffer_ready_cb,
