@@ -23,7 +23,7 @@
 #include <config.h>
 #include <glib.h>
 #define GDK_PIXBUF_ENABLE_BACKEND
-#include <gdk-pixbuf/gdk-pixbuf-io.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <exiv2/basicio.hpp>
 #include <exiv2/error.hpp>
 #include <exiv2/image.hpp>
@@ -57,6 +57,14 @@ const char *_DATE_TAG_NAMES[] = {
 	NULL
 };
 
+const char *_LAST_DATE_TAG_NAMES[] = {
+	"Exif::Image::DateTime",
+	"Xmp::exif::DateTime",
+	"Xmp::xmp::ModifyDate",
+	"Xmp::xmp::MetadataDate",
+	NULL
+};
+
 const char *_ORIGINAL_DATE_TAG_NAMES[] = {
 	"Exif::Photo::DateTimeOriginal",
 	"Xmp::exif::DateTimeOriginal",
@@ -239,11 +247,20 @@ set_file_info (GFileInfo  *info,
 		metadata_info = gth_main_register_metadata_info (&info);
 	}
 
+	if ((metadata_info != NULL) && (metadata_info->type == NULL) && (type_name != NULL))
+		metadata_info->type = g_strdup (type_name);
+
 	if ((metadata_info != NULL) && (metadata_info->display_name == NULL) && (description_utf8 != NULL))
 		metadata_info->display_name = g_strdup (description_utf8);
 
 	metadata = gth_metadata_new ();
-	g_object_set (metadata, "id", key, "description", description_utf8, "formatted", formatted_value_utf8, "raw", raw_value, NULL);
+	g_object_set (metadata,
+		      "id", key,
+		      "description", description_utf8,
+		      "formatted", formatted_value_utf8,
+		      "raw", raw_value,
+		      "value-type", type_name,
+		      NULL);
 	g_file_info_set_attribute_object (info, attribute, G_OBJECT (metadata));
 
 	g_object_unref (metadata);
@@ -264,6 +281,7 @@ set_attribute_from_tagset (GFileInfo  *info,
 	char    *description;
 	char    *formatted_value;
 	char    *raw_value;
+	char    *type_name;
 
 	metadata = NULL;
 	for (i = 0; tagset[i] != NULL; i++) {
@@ -280,8 +298,15 @@ set_attribute_from_tagset (GFileInfo  *info,
 		      "description", &description,
 		      "formatted", &formatted_value,
 		      "raw", &raw_value,
+		      "value-type", &type_name,
 		      NULL);
-	set_file_info (info, attribute, description, formatted_value, raw_value, NULL, NULL);
+	set_file_info (info,
+		       attribute,
+		       description,
+		       formatted_value,
+		       raw_value,
+		       NULL,
+		       type_name);
 }
 
 
@@ -319,7 +344,10 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 static void
 set_attributes_from_tagsets (GFileInfo *info)
 {
-	set_attribute_from_tagset (info, "general::datetime", _DATE_TAG_NAMES);
+	set_attribute_from_tagset (info, "general::datetime", _LAST_DATE_TAG_NAMES);
+	if (g_file_info_get_attribute_object (info, "general::datetime") == NULL)
+		set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
+
 	set_attribute_from_tagset (info, "general::description", _DESCRIPTION_TAG_NAMES);
 	set_attribute_from_tagset (info, "general::title", _TITLE_TAG_NAMES);
 	set_attribute_from_tagset (info, "general::location", _LOCATION_TAG_NAMES);
@@ -360,24 +388,24 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 	if (! exifData.empty()) {
 		Exiv2::ExifData::const_iterator end = exifData.end();
 		for (Exiv2::ExifData::const_iterator md = exifData.begin(); md != end; ++md) {
-			stringstream value;
-			value << *md;
+			stringstream raw_value;
+			raw_value << md->value();
 
-			stringstream short_name;
-			if (md->ifdId () > Exiv2::ifd1Id) {
+			stringstream description;
+			if (! md->tagLabel().empty())
+				description << md->tagLabel();
+			else if (md->ifdId () > Exiv2::ifd1Id)
 				// Must be a MakerNote - include group name
-				short_name << md->groupName() << "." << md->tagName();
-			}
-			else {
+				description << md->groupName() << "." << md->tagName();
+			else
 				// Normal exif tag - just use tag name
-				short_name << md->tagName();
-			}
+				description << md->tagName();
 
 			set_file_info (info,
 				       md->key().c_str(),
-				       short_name.str().c_str(),
-				       value.str().c_str(),
-				       md->toString().c_str(),
+				       description.str().c_str(),
+				       md->print(&exifData).c_str(),
+				       raw_value.str().c_str(),
 				       get_exif_default_category (*md),
 				       md->typeName());
 		}
@@ -387,17 +415,20 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 	if (! iptcData.empty()) {
 		Exiv2::IptcData::iterator end = iptcData.end();
 		for (Exiv2::IptcData::iterator md = iptcData.begin(); md != end; ++md) {
-			stringstream value;
-			value << *md;
+			stringstream raw_value;
+			raw_value << md->value();
 
-			stringstream short_name;
-			short_name << md->tagName();
+			stringstream description;
+			if (! md->tagLabel().empty())
+				description << md->tagLabel();
+			else
+				description << md->tagName();
 
 			set_file_info (info,
 				       md->key().c_str(),
-				       short_name.str().c_str(),
-				       value.str().c_str(),
-				       md->toString().c_str(),
+				       description.str().c_str(),
+				       md->print().c_str(),
+				       raw_value.str().c_str(),
 				       "Iptc",
 				       md->typeName());
 		}
@@ -407,17 +438,20 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 	if (! xmpData.empty()) {
 		Exiv2::XmpData::iterator end = xmpData.end();
 		for (Exiv2::XmpData::iterator md = xmpData.begin(); md != end; ++md) {
-			stringstream value;
-			value << *md;
+			stringstream raw_value;
+			raw_value << md->value();
 
-			stringstream short_name;
-			short_name << md->groupName() << "." << md->tagName();
+			stringstream description;
+			if (! md->tagLabel().empty())
+				description << md->tagLabel();
+			else
+				description << md->groupName() << "." << md->tagName();
 
 			set_file_info (info,
 				       md->key().c_str(),
-				       short_name.str().c_str(),
-				       value.str().c_str(),
-				       md->toString().c_str(),
+				       description.str().c_str(),
+				       md->print().c_str(),
+				       raw_value.str().c_str(),
 				       "Xmp::Embedded",
 				       md->typeName());
 		}
@@ -523,17 +557,20 @@ exiv2_read_sidecar (GFile     *file,
 		if (! xmpData.empty()) {
 			Exiv2::XmpData::iterator end = xmpData.end();
 			for (Exiv2::XmpData::iterator md = xmpData.begin(); md != end; ++md) {
-				stringstream value;
-				value << *md;
+				stringstream raw_value;
+				raw_value << md->value();
 
-				stringstream short_name;
-				short_name << md->groupName() << "." << md->tagName();
+				stringstream description;
+				if (! md->tagLabel().empty())
+					description << md->tagLabel();
+				else
+					description << md->groupName() << "." << md->tagName();
 
 				set_file_info (info,
 					       md->key().c_str(),
-					       short_name.str().c_str(),
-					       value.str().c_str(),
-					       md->toString().c_str(),
+					       description.str().c_str(),
+					       md->print().c_str(),
+					       raw_value.str().c_str(),
 					       "Xmp::Sidecar",
 					       md->typeName());
 			}
@@ -573,6 +610,28 @@ mandatory_string (Exiv2::ExifData &checkdata,
 }
 
 
+const char *
+gth_main_get_metadata_type (GthMetadata *metadata,
+			    const char  *attribute)
+{
+	const char      *value_type;
+	GthMetadataInfo *metadatum_info;
+
+	value_type = gth_metadata_get_value_type (metadata);
+	if (g_strcmp0 (value_type, "Undefined") == 0)
+		value_type = NULL;
+
+	if (value_type != NULL)
+		return value_type;
+
+	metadatum_info = gth_main_get_metadata_info (attribute);
+	if (metadatum_info != NULL)
+		value_type = metadatum_info->type;
+
+	return value_type;
+}
+
+
 static Exiv2::DataBuf
 exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 			      GFileInfo             *info,
@@ -581,11 +640,11 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 	char **attributes;
 	int    i;
 
-	image->readMetadata();
+	image->clearMetadata();
 
 	// EXIF Data
 
-	Exiv2::ExifData &ed = image->exifData();
+	Exiv2::ExifData ed;
 	attributes = g_file_info_list_attributes (info, "Exif");
 	for (i = 0; attributes[i] != NULL; i++) {
 		GthMetadata *metadatum;
@@ -598,21 +657,23 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 			/* If the metadatum has no value yet, a new empty value
 			 * is created. The type is taken from Exiv2's tag
 			 * lookup tables. If the tag is not found in the table,
-			 * the type defaults to Ascii.
-			 * We always create the metadatum explicilty if the
+			 * the type defaults to ASCII.
+			 * We always create the metadatum explicitly if the
 			 * type is available to avoid type errors.
 			 * See bug #610389 for more details.  The original
-			 * expanation is here:
+			 * explanation is here:
 			 * http://uk.groups.yahoo.com/group/exiv2/message/1472
 			 */
 
-			GthMetadataInfo *metadatum_info = gth_main_get_metadata_info (attributes[i]);
-			if ((metadatum_info != NULL) && (metadatum_info->type != NULL)) {
-				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (metadatum_info->type));
+			const char *raw_value = gth_metadata_get_raw (metadatum);
+			const char *value_type = gth_main_get_metadata_type (metadatum, attributes[i]);
+
+			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) && (value_type != NULL)) {
+				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (value_type));
+				value->read (raw_value);
 				Exiv2::ExifKey exif_key(key);
 				ed.add (exif_key, value.get());
 			}
-			ed[key] = gth_metadata_get_raw (metadatum);
 		}
 		catch (Exiv2::AnyError& e) {
 			/* we don't care about invalid key errors */
@@ -657,7 +718,7 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 
 	// Update the thumbnail
 
-	Exiv2::ExifThumb thumb(image->exifData());
+	Exiv2::ExifThumb thumb(ed);
 	if ((pixbuf != NULL) && (width > 0) && (height > 0)) {
 		GdkPixbuf *thumb_pixbuf;
 		char      *buffer;
@@ -692,21 +753,23 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 
 	// IPTC Data
 
-	Exiv2::IptcData &id = image->iptcData();
+	Exiv2::IptcData id;
 	attributes = g_file_info_list_attributes (info, "Iptc");
 	for (i = 0; attributes[i] != NULL; i++) {
 		GthMetadata *metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
 		char *key = exiv2_key_from_attribute (attributes[i]);
 
 		try {
-			/* See the exif data code above for an explanation. */
-			GthMetadataInfo *metadatum_info = gth_main_get_metadata_info (attributes[i]);
-			if ((metadatum_info != NULL) && (metadatum_info->type != NULL)) {
-				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (metadatum_info->type));
+			const char *raw_value = gth_metadata_get_raw (metadatum);
+			const char *value_type = gth_main_get_metadata_type (metadatum, attributes[i]);
+
+			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) &&  (value_type != NULL)) {
+				/* See the exif data code above for an explanation. */
+				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (value_type));
+				value->read (raw_value);
 				Exiv2::IptcKey iptc_key(key);
 				id.add (iptc_key, value.get());
 			}
-			id[key] = gth_metadata_get_raw (metadatum);
 		}
 		catch (Exiv2::AnyError& e) {
 			/* we don't care about invalid key errors */
@@ -719,7 +782,7 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 
 	// XMP Data
 
-	Exiv2::XmpData &xd = image->xmpData();
+	Exiv2::XmpData xd;
 	attributes = g_file_info_list_attributes (info, "Xmp");
 	for (i = 0; attributes[i] != NULL; i++) {
 		GthMetadata *metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
@@ -733,17 +796,15 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 			xd.erase (iter);
 
 		try {
-			const char *value = gth_metadata_get_raw (metadatum);
+			const char *raw_value = gth_metadata_get_raw (metadatum);
+			const char *value_type = gth_main_get_metadata_type (metadatum, attributes[i]);
 
-			if ((value != NULL) && strcmp (value, "") != 0) {
+			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) &&  (value_type != NULL)) {
 				/* See the exif data code above for an explanation. */
-				GthMetadataInfo *metadatum_info = gth_main_get_metadata_info (attributes[i]);
-				if ((metadatum_info != NULL) && (metadatum_info->type != NULL)) {
-					Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (metadatum_info->type));
-					Exiv2::XmpKey xmp_key(key);
-					xd.add (xmp_key, value.get());
-				}
-				xd[key] = gth_metadata_get_raw (metadatum);
+				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (value_type));
+				value->read (raw_value);
+				Exiv2::XmpKey xmp_key(key);
+				xd.add (xmp_key, value.get());
 			}
 		}
 		catch (Exiv2::AnyError& e) {
@@ -755,7 +816,16 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 	}
 	g_strfreev (attributes);
 
-	image->writeMetadata();
+	try {
+		image->setExifData(ed);
+		image->setIptcData(id);
+		image->setXmpData(xd);
+		image->writeMetadata();
+	}
+	catch (Exiv2::AnyError& e) {
+		g_warning ("%s", e.what());
+	}
+
 	Exiv2::BasicIo &io = image->io();
 	io.open();
 
@@ -767,8 +837,11 @@ extern "C"
 gboolean
 exiv2_supports_writes (const char *mime_type)
 {
-	return (_g_content_type_is_a (mime_type, "image/jpeg") ||
-		_g_content_type_is_a (mime_type, "image/png"));
+	return (g_content_type_equals (mime_type, "image/jpeg")
+#if HAVE_EXIV2_020
+		|| g_content_type_equals (mime_type, "image/tiff")
+#endif
+		|| g_content_type_equals (mime_type, "image/png"));
 }
 
 
