@@ -501,7 +501,7 @@ complete_authorization (FacebookAuthentication *self)
 			  G_CALLBACK (complete_authorization_messagedialog_response_cb),
 			  self);
 
-	if (GTK_WIDGET_VISIBLE (self->priv->dialog))
+	if (gtk_widget_get_visible (self->priv->dialog))
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->dialog));
 	else
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->browser));
@@ -533,15 +533,16 @@ ask_authorization_messagedialog_response_cb (GtkDialog *dialog,
 
 	case GTK_RESPONSE_OK:
 		{
-			char   *url;
-			GError *error = NULL;
+			GdkScreen *screen;
+			char      *url;
+			GError    *error = NULL;
 
+			screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 
 			url = facebook_connection_get_login_link (self->priv->conn, FACEBOOK_ACCESS_WRITE);
-			if (gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)), url, 0, &error)) {
+			if (gtk_show_uri (screen, url, 0, &error))
 				complete_authorization (self);
-			}
 			else
 				show_authentication_error_dialog (self, &error);
 
@@ -567,7 +568,7 @@ ask_authorization (FacebookAuthentication *self)
 
 	builder = _gtk_builder_new_from_file ("facebook-ask-authorization.ui", "facebook");
 	dialog = _gtk_builder_get_widget (builder, "ask_authorization_messagedialog");
-	if (GTK_WIDGET_VISIBLE (self->priv->dialog))
+	if (gtk_widget_get_visible (self->priv->dialog))
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->dialog));
 	else
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->browser));
