@@ -31,6 +31,7 @@
 #include "facebook-service.h"
 #include "facebook-user.h"
 
+#define FACEBOOK_MAX_IMAGE_SIZE 720
 
 typedef struct {
 	FacebookAlbum       *album;
@@ -683,6 +684,7 @@ upload_photo_file_buffer_ready_cb (void     **buffer,
 	FacebookService *self = user_data;
 	GthFileData     *file_data;
 	SoupMultipart   *multipart;
+	GthPixbufSaver  *saver;
 	char            *uri;
 	SoupBuffer      *body;
 	SoupMessage     *msg;
@@ -730,7 +732,64 @@ upload_photo_file_buffer_ready_cb (void     **buffer,
 		g_hash_table_unref (data_set);
 	}
 
-	/* the file part */
+	/* the file part: rotate and scale the image if required */
+
+	saver = gth_main_get_pixbuf_saver (gth_file_data_get_mime_type (file_data));
+	if (saver != NULL) {
+		GInputStream *stream;
+		GdkPixbuf    *pixbuf;
+		GdkPixbuf    *tmp_pixbuf;
+		int           width;
+		int           height;
+
+		stream = g_memory_input_stream_new_from_data (*buffer, count, NULL);
+		pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, &error);
+		if (pixbuf == NULL) {
+			g_object_unref (stream);
+			soup_multipart_free (multipart);
+			upload_photos_done (self, error);
+			return;
+		}
+
+		g_object_unref (stream);
+
+		tmp_pixbuf = gdk_pixbuf_apply_embedded_orientation (pixbuf);
+		g_object_unref (pixbuf);
+		pixbuf = tmp_pixbuf;
+
+		width = gdk_pixbuf_get_width (pixbuf);
+		height = gdk_pixbuf_get_height (pixbuf);
+		if (scale_keeping_ratio (&width,
+					 &height,
+					 FACEBOOK_MAX_IMAGE_SIZE,
+					 FACEBOOK_MAX_IMAGE_SIZE,
+					 FALSE))
+		{
+			tmp_pixbuf = _gdk_pixbuf_scale_simple_safe (pixbuf, width, height, GDK_INTERP_BILINEAR);
+			g_object_unref (pixbuf);
+			pixbuf = tmp_pixbuf;
+		}
+
+		g_free (*buffer);
+		*buffer = NULL;
+
+		if (! gth_pixbuf_saver_save_pixbuf (saver,
+						    pixbuf,
+						    (char **)buffer,
+						    &count,
+						    gth_file_data_get_mime_type (file_data),
+						    &error))
+		{
+			g_object_unref (pixbuf);
+			g_object_unref (saver);
+			soup_multipart_free (multipart);
+			upload_photos_done (self, error);
+			return;
+		}
+
+		g_object_unref (pixbuf);
+		g_object_unref (saver);
+	}
 
 	uri = g_file_get_uri (file_data->file);
 	body = soup_buffer_new (SOUP_MEMORY_TEMPORARY, *buffer, count);
@@ -837,8 +896,7 @@ facebook_service_upload_photos (FacebookService     *self,
 	self->priv->post_photos->n_files = 0;
 
 	_g_query_all_metadata_async (file_list,
-				     FALSE,
-				     TRUE,
+				     GTH_LIST_DEFAULT,
 				     "*",
 				     self->priv->post_photos->cancellable,
 				     upload_photos_info_ready_cb,
