@@ -143,11 +143,16 @@ clipboard_get_cb (GtkClipboard     *clipboard,
 		  gpointer          user_data_or_owner)
 {
 	ClipboardData *clipboard_data = user_data_or_owner;
+	GdkAtom        targets[1];
+	int            n_targets;
 
-	if (gtk_targets_include_uri (&selection_data->target, 1)) {
+	targets[0] = gtk_selection_data_get_target (selection_data);
+	n_targets = 1;
+
+	if (gtk_targets_include_uri (targets, n_targets)) {
 		gtk_selection_data_set_uris (selection_data, clipboard_data->uris);
 	}
-	else if (gtk_targets_include_text (&selection_data->target, 1)) {
+	else if (gtk_targets_include_text (targets, n_targets)) {
 		char  *str;
 		gsize  len;
 
@@ -155,7 +160,7 @@ clipboard_get_cb (GtkClipboard     *clipboard,
 		gtk_selection_data_set_text (selection_data, str, len);
 		g_free (str);
 	}
-	else if (selection_data->target == GNOME_COPIED_FILES) {
+	else if (gtk_selection_data_get_target (selection_data) == GNOME_COPIED_FILES) {
 		char  *str;
 		gsize  len;
 
@@ -278,12 +283,19 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 {
 	PasteData   *paste_data = user_data;
 	GthBrowser  *browser = paste_data->browser;
+	const char  *raw_data;
 	char       **clipboard_data;
 	int          i;
 	GthTask     *task;
 
-	clipboard_data = g_strsplit_set ((const char *) gtk_selection_data_get_data (selection_data), "\n\r", -1);
-	if (clipboard_data[0] == NULL) {
+	raw_data = (const char *) gtk_selection_data_get_data (selection_data);
+	if (raw_data == NULL) {
+		paste_data_free (paste_data);
+		return;
+	}
+
+	clipboard_data = g_strsplit_set (raw_data, "\n\r", -1);
+	if ((clipboard_data == NULL) || (clipboard_data[0] == NULL)) {
 		g_strfreev (clipboard_data);
 		paste_data_free (paste_data);
 		return;
@@ -337,6 +349,9 @@ gth_browser_activate_action_edit_paste (GtkAction  *action,
 {
 	PasteData *paste_data;
 
+	if (gtk_window_get_focus (GTK_WINDOW (browser)) != gth_browser_get_file_list_view (browser))
+		return;
+
 	paste_data = g_new0 (PasteData, 1);
 	paste_data->browser = g_object_ref (browser);
 	paste_data->destination = g_object_ref (gth_browser_get_location_data (browser));
