@@ -106,9 +106,11 @@ completed_messagedialog_response_cb (GtkDialog *dialog,
 
 	case _OPEN_IN_BROWSER_RESPONSE:
 		{
-			char   *url = NULL;
-			GError *error = NULL;
+			GdkScreen *screen;
+			char      *url = NULL;
+			GError    *error = NULL;
 
+			screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 
 			if (data->photoset == NULL) {
@@ -130,7 +132,7 @@ completed_messagedialog_response_cb (GtkDialog *dialog,
 			else if (data->photoset->id != NULL)
 				url = g_strconcat (data->server->url, "/photos/", data->user->id, "/sets/", data->photoset->id, NULL);
 
-			if ((url != NULL) && ! gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)), url, 0, &error)) {
+			if ((url != NULL) && ! gtk_show_uri (screen, url, 0, &error)) {
 				if (data->conn != NULL)
 					gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
