@@ -33,7 +33,7 @@
 
 struct _GthImageViewerPagePrivate {
 	GthBrowser        *browser;
-	GtkWidget         *nav_window;
+	GtkWidget         *image_navigator;
 	GtkWidget         *viewer;
 	GthImagePreloader *preloader;
 	GtkActionGroup    *actions;
@@ -238,6 +238,15 @@ viewer_button_press_event_cb (GtkWidget          *widget,
 }
 
 
+static gboolean
+viewer_popup_menu_cb (GtkWidget          *widget,
+		      GthImageViewerPage *self)
+{
+	gth_browser_file_menu_popup (self->priv->browser, NULL);
+	return TRUE;
+}
+
+
 static gboolean
 viewer_scroll_event_cb (GtkWidget 	   *widget,
 		        GdkEventScroll      *event,
@@ -430,6 +439,10 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 			  "zoom_changed",
 			  G_CALLBACK (viewer_zoom_changed_cb),
 			  self);
+	g_signal_connect (G_OBJECT (self->priv->viewer),
+			  "popup-menu",
+			  G_CALLBACK (viewer_popup_menu_cb),
+			  self);
 	g_signal_connect_after (G_OBJECT (self->priv->viewer),
 				"button_press_event",
 				G_CALLBACK (viewer_button_press_event_cb),
@@ -447,10 +460,10 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 			  G_CALLBACK (viewer_key_press_cb),
 			  self);
 
-	self->priv->nav_window = gth_nav_window_new (GTH_IMAGE_VIEWER (self->priv->viewer));
-	gtk_widget_show (self->priv->nav_window);
+	self->priv->image_navigator = gth_image_navigator_new (GTH_IMAGE_VIEWER (self->priv->viewer));
+	gtk_widget_show (self->priv->image_navigator);
 
-	gth_browser_set_viewer_widget (browser, self->priv->nav_window);
+	gth_browser_set_viewer_widget (browser, self->priv->image_navigator);
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 
 	/* gconf notifications */
@@ -512,8 +525,13 @@ gth_image_viewer_page_real_deactivate (GthViewerPage *base)
 
 	/**/
 
+	gtk_ui_manager_remove_action_group (gth_browser_get_ui_manager (self->priv->browser), self->priv->actions);
+	g_object_unref (self->priv->actions);
+	self->priv->actions = NULL;
+
 	g_signal_handler_disconnect (self->priv->preloader, self->priv->preloader_sig_id);
 	self->priv->preloader_sig_id = 0;
+
 	g_object_unref (self->priv->preloader);
 	self->priv->preloader = NULL;
 
@@ -589,6 +607,7 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	    && (gth_file_data_get_mtime (file_data) == gth_file_data_get_mtime (self->priv->file_data))
 	    && ! self->priv->pixbuf_changed)
 	{
+		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), TRUE);
 		return;
 	}
 
@@ -621,7 +640,7 @@ gth_image_viewer_page_real_focus (GthViewerPage *base)
 	GtkWidget *widget;
 
 	widget = GTH_IMAGE_VIEWER_PAGE (base)->priv->viewer;
-	if (GTK_WIDGET_REALIZED (widget) && GTK_WIDGET_MAPPED (widget))
+	if (gtk_widget_get_realized (widget) && gtk_widget_get_mapped (widget))
 		gtk_widget_grab_focus (widget);
 }
 
@@ -634,11 +653,11 @@ gth_image_viewer_page_real_fullscreen (GthViewerPage *base,
 
 	self = (GthImageViewerPage *) base;
 	if (active) {
-		gth_nav_window_set_scrollbars_visible (GTH_NAV_WINDOW (self->priv->nav_window), FALSE);
+		gth_image_navigator_set_scrollbars_visible (GTH_IMAGE_NAVIGATOR (self->priv->image_navigator), FALSE);
 		gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
 	}
 	else {
-		gth_nav_window_set_scrollbars_visible (GTH_NAV_WINDOW (self->priv->nav_window), TRUE);
+		gth_image_navigator_set_scrollbars_visible (GTH_IMAGE_NAVIGATOR (self->priv->image_navigator), TRUE);
 		gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_boolean (PREF_BLACK_BACKGROUND, FALSE));
 	}
 }
@@ -922,7 +941,7 @@ _gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
 	g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
 	g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
 
-	size = g_strdup_printf ("%d x %d", width, height);
+	size = g_strdup_printf ("%d × %d", width, height);
 	g_file_info_set_attribute_string (file_data->info, "general::dimensions", size);
 
 	gth_monitor_metadata_changed (gth_main_get_default_monitor (), file_data);
@@ -1099,7 +1118,7 @@ add_non_content_height (GthImageViewerPage *self,
 {
 	int height = 0;
 
-	if ((non_content != NULL) && GTK_WIDGET_VISIBLE (non_content)) {
+	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
 		GtkAllocation allocation;
 		gtk_widget_get_allocation (non_content, &allocation);
 		height = allocation.height;
