@@ -233,8 +233,10 @@ list_tools__gth_browser_construct_cb (GthBrowser *browser)
 
 	/* tools menu button */
 
-	tool_item = gth_toggle_menu_tool_button_new_from_stock (GTK_STOCK_EXECUTE);
-	gtk_tool_button_set_label_widget (GTK_TOOL_BUTTON (tool_item), _gtk_create_toggle_menu_tool_label (_("Tools")));
+	tool_item = g_object_new (GTH_TYPE_TOGGLE_MENU_TOOL_BUTTON,
+				  "stock-id", GTK_STOCK_EXECUTE,
+				  "label", _("Tools"),
+				  NULL);
 	gtk_widget_set_tooltip_text (GTK_WIDGET (tool_item), _("Batch tools for multiple files"));
 	gth_toggle_menu_tool_button_set_menu (GTH_TOGGLE_MENU_TOOL_BUTTON (tool_item),
 					      gtk_ui_manager_get_widget (gth_browser_get_ui_manager (browser), "/ListToolsPopup"));
@@ -242,8 +244,10 @@ list_tools__gth_browser_construct_cb (GthBrowser *browser)
 	gtk_widget_show (GTK_WIDGET (tool_item));
 	gtk_toolbar_insert (GTK_TOOLBAR (gth_browser_get_browser_toolbar (browser)), tool_item, -1);
 
-	tool_item = gth_toggle_menu_tool_button_new_from_stock (GTK_STOCK_EXECUTE);
-	gtk_tool_button_set_label_widget (GTK_TOOL_BUTTON (tool_item), _gtk_create_toggle_menu_tool_label (_("Tools")));
+	tool_item = g_object_new (GTH_TYPE_TOGGLE_MENU_TOOL_BUTTON,
+				  "stock-id", GTK_STOCK_EXECUTE,
+				  "label", _("Tools"),
+				  NULL);
 	gtk_widget_set_tooltip_text (GTK_WIDGET (tool_item), _("Batch tools for multiple files"));
 	gth_toggle_menu_tool_button_set_menu (GTH_TOGGLE_MENU_TOOL_BUTTON (tool_item),
 					      gtk_ui_manager_get_widget (gth_browser_get_ui_manager (browser), "/ListToolsPopup"));
@@ -316,17 +320,14 @@ list_tools__gth_browser_file_list_key_press_cb (GthBrowser  *browser,
 						GdkEventKey *event)
 {
 	gpointer  result = NULL;
-	guint     keyval;
 	GList    *script_list;
 	GList    *scan;
 
-	keyval = gdk_keyval_to_lower (event->keyval);
-
 	script_list = gth_script_file_get_scripts (gth_script_file_get ());
 	for (scan = script_list; scan; scan = scan->next) {
 		GthScript *script = scan->data;
 
-		if (gth_script_get_shortcut (script) == keyval) {
+		if (gth_script_get_shortcut (script) == event->keyval) {
 			exec_script (browser, script);
 			result = GINT_TO_POINTER (1);
 			break;
