@@ -25,7 +25,9 @@
 
 #include <gthumb.h>
 
-void list_tools__gth_browser_construct_cb          (GthBrowser *browser);
-void list_tools__gth_browser_update_sensitivity_cb (GthBrowser *browser);
+void      list_tools__gth_browser_construct_cb           (GthBrowser  *browser);
+void      list_tools__gth_browser_update_sensitivity_cb  (GthBrowser  *browser);
+gpointer  list_tools__gth_browser_file_list_key_press_cb (GthBrowser  *browser,
+							  GdkEventKey *event);
 
 #endif /* CALLBACKS_H */
