@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <gthumb.h>
+#include <gdk/gdkkeysyms.h>
 #include "gth-script.h"
 
 
@@ -33,7 +34,8 @@ enum {
         PROP_VISIBLE,
         PROP_SHELL_SCRIPT,
         PROP_FOR_EACH_FILE,
-        PROP_WAIT_COMMAND
+        PROP_WAIT_COMMAND,
+        PROP_SHORTCUT
 };
 
 
@@ -45,6 +47,7 @@ struct _GthScriptPrivate {
 	gboolean  shell_script;
 	gboolean  for_each_file;
 	gboolean  wait_command;
+	guint     shortcut;
 };
 
 
@@ -68,6 +71,7 @@ gth_script_real_create_element (DomDomizable *base,
 					       "shell-script", (self->priv->shell_script ? "true" : "false"),
 					       "for-each-file", (self->priv->for_each_file ? "true" : "false"),
 					       "wait-command", (self->priv->wait_command ? "true" : "false"),
+					       "shortcut", gdk_keyval_name (self->priv->shortcut),
 					       NULL);
 	if (! self->priv->visible)
 		dom_element_set_attribute (element, "display", "none");
@@ -76,6 +80,16 @@ gth_script_real_create_element (DomDomizable *base,
 }
 
 
+static guint
+_gdk_keyval_from_name (const gchar *keyval_name)
+{
+	if (keyval_name != NULL)
+		return gdk_keyval_from_name (keyval_name);
+	else
+		return GDK_VoidSymbol;
+}
+
+
 static void
 gth_script_real_load_from_element (DomDomizable *base,
 				   DomElement   *element)
@@ -93,6 +107,7 @@ gth_script_real_load_from_element (DomDomizable *base,
 		      "shell-script", (g_strcmp0 (dom_element_get_attribute (element, "shell-script"), "true") == 0),
 		      "for-each-file", (g_strcmp0 (dom_element_get_attribute (element, "for-each-file"), "true") == 0),
 		      "wait-command", (g_strcmp0 (dom_element_get_attribute (element, "wait-command"), "true") == 0),
+		      "shortcut", _gdk_keyval_from_name (dom_element_get_attribute (element, "shortcut")),
 		      NULL);
 }
 
@@ -112,6 +127,7 @@ gth_script_real_duplicate (GthDuplicable *duplicable)
 		      "shell-script", script->priv->shell_script,
 		      "for-each-file", script->priv->for_each_file,
 		      "wait-command", script->priv->wait_command,
+		      "shortcut", script->priv->shortcut,
 		      NULL);
 
 	return (GObject *) new_script;
@@ -174,6 +190,9 @@ gth_script_set_property (GObject      *object,
 	case PROP_WAIT_COMMAND:
 		self->priv->wait_command = g_value_get_boolean (value);
 		break;
+	case PROP_SHORTCUT:
+		self->priv->shortcut = g_value_get_uint (value);
+		break;
 	default:
 		break;
 	}
@@ -212,6 +231,9 @@ gth_script_get_property (GObject    *object,
 	case PROP_WAIT_COMMAND:
 		g_value_set_boolean (value, self->priv->wait_command);
 		break;
+	case PROP_SHORTCUT:
+		g_value_set_uint (value, self->priv->wait_command);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -283,6 +305,15 @@ gth_script_class_init (GthScriptClass *klass)
 							       "Whether to wait command to finish",
 							       FALSE,
 							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_SHORTCUT,
+					 g_param_spec_uint ("shortcut",
+							    "Shortcut",
+							    "The keyboard shortcut to activate the script",
+							    0,
+							    G_MAXUINT,
+							    GDK_VoidSymbol,
+							    G_PARAM_READWRITE));
 }
 
 
@@ -515,17 +546,17 @@ get_basename_func (GthFileData *file_data)
 
 
 static char *
-get_name_wo_ext_func (GthFileData *file_data)
+get_basename_wo_ext_func (GthFileData *file_data)
 {
-	char *path;
-	char *name;
+	char *basename;
+	char *basename_wo_ext;
 
-	path = g_file_get_path (file_data->file);
-	name = _g_uri_remove_extension (path);
+	basename = g_file_get_basename (file_data->file);
+	basename_wo_ext = _g_uri_remove_extension (basename);
 
-	g_free (path);
+	g_free (basename);
 
-	return name;
+	return basename_wo_ext;
 }
 
 
@@ -624,7 +655,7 @@ ask_value (ReplaceData  *replace_data,
 		gtk_widget_hide (_gtk_builder_get_widget (builder, "skip_button"));
 
 	g_object_ref (builder);
-	thumb_loader = gth_thumb_loader_new (128, 128);
+	thumb_loader = gth_thumb_loader_new (128);
 	g_signal_connect (thumb_loader, "ready", G_CALLBACK (thumb_loader_ready_cb), builder);
 	gth_thumb_loader_set_file (thumb_loader, file_data);
 	gth_thumb_loader_load (thumb_loader);
@@ -730,7 +761,7 @@ command_line_eval_cb (const GMatchInfo *info,
 	else if (strcmp (match, "%B") == 0)
 		r = create_file_list (replace_data->file_list, get_basename_func, replace_data->quote_values);
 	else if (strcmp (match, "%N") == 0)
-		r = create_file_list (replace_data->file_list, get_name_wo_ext_func, replace_data->quote_values);
+		r = create_file_list (replace_data->file_list, get_basename_wo_ext_func, replace_data->quote_values);
 	else if (strcmp (match, "%E") == 0)
 		r = create_file_list (replace_data->file_list, get_ext_func, replace_data->quote_values);
 	else if (strcmp (match, "%P") == 0)
@@ -813,3 +844,10 @@ gth_script_get_command_line (GthScript  *script,
 
 	return result;
 }
+
+
+guint
+gth_script_get_shortcut (GthScript *script)
+{
+	return script->priv->shortcut;
+}
