@@ -452,7 +452,7 @@ complete_authorization (OAuthAuthentication *self)
 			  G_CALLBACK (complete_authorization_messagedialog_response_cb),
 			  self);
 
-	if (GTK_WIDGET_VISIBLE (self->priv->dialog))
+	if (gtk_widget_get_visible (self->priv->dialog))
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->dialog));
 	else
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->browser));
@@ -484,13 +484,15 @@ ask_authorization_messagedialog_response_cb (GtkDialog *dialog,
 
 	case GTK_RESPONSE_OK:
 		{
-			char   *url;
-			GError *error = NULL;
+			GdkScreen *screen;
+			char      *url;
+			GError    *error = NULL;
 
+			screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 
 			url = oauth_connection_get_login_link (self->priv->conn);
-			if (gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)), url, 0, &error))
+			if (gtk_show_uri (screen, url, 0, &error))
 				complete_authorization (self);
 			else
 				show_authentication_error_dialog (self, &error);
@@ -527,7 +529,7 @@ ask_authorization (OAuthAuthentication *self)
 			  self);
 
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	if (GTK_WIDGET_VISIBLE (self->priv->dialog))
+	if (gtk_widget_get_visible (self->priv->dialog))
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->dialog));
 	else
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (self->priv->browser));
