@@ -102,7 +102,7 @@ import_dialog_destroy_cb (GtkWidget  *widget,
 }
 
 
-GList *
+static GList *
 get_files_to_download (DialogData *data)
 {
 	GthFileView *file_view;
@@ -274,7 +274,7 @@ show_import_dialog (DialogData *data)
 {
 	update_account_list (data);
 	update_album_list (data);
-	gth_task_dialog (GTH_TASK (data->conn), TRUE);
+	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (data->browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
@@ -295,7 +295,7 @@ list_albums_ready_cb (GObject      *source_object,
 	data->albums = picasa_web_service_list_albums_finish (picasaweb, result, &error);
 	if (error != NULL) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the album list"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -312,7 +312,7 @@ get_album_list (DialogData *data)
 {
 	if (data->picasaweb == NULL)
 		data->picasaweb = picasa_web_service_new (data->conn);
-	gth_task_dialog (GTH_TASK (data->conn), FALSE);
+	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 	picasa_web_service_list_albums (data->picasaweb,
 				        "default",
 				        data->cancellable,
@@ -354,7 +354,7 @@ connection_ready_cb (GObject      *source_object,
 		}
 		else {
 			if (data->conn != NULL)
-				gth_task_dialog (GTH_TASK (data->conn), TRUE);
+				gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
 			gtk_widget_destroy (data->dialog);
 		}
@@ -430,7 +430,7 @@ account_properties_dialog (DialogData *data,
 	GtkWidget *dialog;
 
 	if (data->conn != NULL)
-		gth_task_dialog (GTH_TASK (data->conn), TRUE);
+		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	dialog = picasa_account_properties_dialog_new (email, NULL, NULL);
 	g_signal_connect (dialog,
@@ -449,11 +449,11 @@ static void
 connect_to_server_step2 (DialogData *data)
 {
 	if (data->password == NULL) {
-		gth_task_dialog (GTH_TASK (data->conn), TRUE);
+		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		account_properties_dialog (data, data->email);
 	}
 	else {
-		gth_task_dialog (GTH_TASK (data->conn), FALSE);
+		gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 		google_connection_connect (data->conn,
 					   data->email,
 					   data->password,
@@ -750,12 +750,13 @@ list_photos_ready_cb (GObject      *source_object,
 	GList            *list;
 	GList            *scan;
 
-	gth_task_dialog (GTH_TASK (data->conn), TRUE);
+	if (data->conn != NULL)
+		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 	_g_object_list_unref (data->photos);
 	data->photos = picasa_web_service_list_albums_finish (picasaweb, result, &error);
 	if (error != NULL) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the photo list"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -799,7 +800,7 @@ album_combobox_changed_cb (GtkComboBox *widget,
 			    ALBUM_DATA_COLUMN, &data->album,
 			    -1);
 
-	gth_task_dialog (GTH_TASK (data->conn), FALSE);
+	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 	picasa_web_service_list_photos (data->picasaweb,
 					data->album,
 					data->cancellable,
@@ -943,9 +944,11 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 						NULL);
 	}
 
+	_gtk_window_resize_to_fit_screen_height (data->dialog, 500);
+
 	/* Set the widget data */
 
-	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL);
+	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL, FALSE);
 	thumb_loader = gth_file_list_get_thumb_loader (GTH_FILE_LIST (data->file_list));
 	gth_thumb_loader_use_cache (thumb_loader, FALSE);
 	gth_thumb_loader_set_loader (thumb_loader, picasa_web_thumbnail_loader);
