@@ -375,10 +375,21 @@ post_photos_done (PicasaWebService *self,
 	GSimpleAsyncResult *result;
 
 	result = google_connection_get_result (self->priv->conn);
-	if (error == NULL)
+	if (error == NULL) {
 		g_simple_async_result_set_op_res_gboolean (result, TRUE);
-	else
+	}
+	else  {
+		if (self->priv->post_photos->current != NULL) {
+			GthFileData *file_data = self->priv->post_photos->current->data;
+			char        *msg;
+
+			msg = g_strdup_printf (_("Could not upload '%s': %s"), g_file_info_get_display_name (file_data->info), error->message);
+			g_free (error->message);
+			error->message = msg;
+		}
 		g_simple_async_result_set_from_error (result, error);
+	}
+
 	g_simple_async_result_complete_in_idle (result);
 }
 
@@ -396,7 +407,7 @@ post_photo_ready_cb (SoupSession *session,
 	if (msg->status_code != 201) {
 		GError *error;
 
-		error = g_error_new (SOUP_HTTP_ERROR, msg->status_code, "%s", soup_status_get_phrase (msg->status_code));
+		error = g_error_new_literal (SOUP_HTTP_ERROR, msg->status_code, soup_status_get_phrase (msg->status_code));
 		post_photos_done (self, error);
 		g_error_free (error);
 
@@ -598,8 +609,7 @@ picasa_web_service_post_photos (PicasaWebService    *self,
 	}
 
 	_g_query_all_metadata_async (file_list,
-				     FALSE,
-				     TRUE,
+				     GTH_LIST_DEFAULT,
 				     "*",
 				     self->priv->post_photos->cancellable,
 				     post_photos_info_ready_cb,
@@ -820,7 +830,7 @@ picasa_web_accounts_save_to_file (GList      *accounts,
 	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, "accounts", "picasaweb.xml", NULL);
 	file = g_file_new_for_path (filename);
 	buffer = dom_document_dump (doc, &len);
-	g_write_file (file, FALSE, G_FILE_CREATE_PRIVATE|G_FILE_CREATE_REPLACE_DESTINATION, buffer, len, NULL, NULL);
+	g_write_file (file, FALSE, G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION, buffer, len, NULL, NULL);
 
 	g_free (buffer);
 	g_object_unref (file);
