@@ -33,12 +33,15 @@ void
 gth_browser_activate_action_view_slideshow (GtkAction  *action,
 					    GthBrowser *browser)
 {
-	GList       *items;
-	GList       *file_list;
-	GtkWidget   *slideshow;
-	GthFileData *location;
-	char        *transition_id;
-	GList       *transitions = NULL;
+	GList        *items;
+	GList        *file_list;
+	GList        *filtered_list;
+	GList        *scan;
+	GthProjector *projector;
+	GtkWidget    *slideshow;
+	GthFileData  *location;
+	char         *transition_id;
+	GList        *transitions = NULL;
 
 	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 	if ((items == NULL) || (items->next == NULL))
@@ -46,20 +49,48 @@ gth_browser_activate_action_view_slideshow (GtkAction  *action,
 	else
 		file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
 
-	slideshow = gth_slideshow_new (browser, file_list);
+	filtered_list = NULL;
+	for (scan = file_list; scan; scan = scan->next) {
+		GthFileData *file_data = scan->data;
+
+		if (_g_mime_type_is_image (gth_file_data_get_mime_type (file_data)))
+			filtered_list = g_list_prepend (filtered_list, g_object_ref (file_data));
+	}
+	filtered_list = g_list_reverse (filtered_list);
+
+	if (filtered_list == NULL) {
+		_g_object_list_unref (file_list);
+		_gtk_tree_path_list_free (items);
+		return;
+	}
 
 	location = gth_browser_get_location_data (browser);
-	if (g_file_info_get_attribute_status (location->info, "slideshow::personalize") == G_FILE_ATTRIBUTE_STATUS_SET) {
+	if (g_file_info_get_attribute_boolean (location->info, "slideshow::personalize"))
+		transition_id = g_strdup (g_file_info_get_attribute_string (location->info, "slideshow::transition"));
+	else
+		transition_id = eel_gconf_get_string (PREF_SLIDESHOW_TRANSITION, DEFAULT_TRANSITION);
+
+	projector = NULL;
+
+#ifdef HAVE_CLUTTER
+	if (ClutterInitResult == CLUTTER_INIT_SUCCESS)
+		projector = &clutter_projector;
+#endif /* HAVE_CLUTTER */
+
+	if ((projector == NULL) || (strcmp (transition_id, "none") == 0))
+		projector = &default_projector;
+
+	slideshow = gth_slideshow_new (projector, browser, filtered_list);
+
+	if (g_file_info_get_attribute_boolean (location->info, "slideshow::personalize")) {
 		gth_slideshow_set_delay (GTH_SLIDESHOW (slideshow), g_file_info_get_attribute_int32 (location->info, "slideshow::delay"));
 		gth_slideshow_set_automatic (GTH_SLIDESHOW (slideshow), g_file_info_get_attribute_boolean (location->info, "slideshow::automatic"));
 		gth_slideshow_set_wrap_around (GTH_SLIDESHOW (slideshow), g_file_info_get_attribute_boolean (location->info, "slideshow::wrap-around"));
-		transition_id = g_strdup (g_file_info_get_attribute_string (location->info, "slideshow::transition"));
 	}
 	else {
 		gth_slideshow_set_delay (GTH_SLIDESHOW (slideshow), (guint) (1000.0 * eel_gconf_get_float (PREF_SLIDESHOW_CHANGE_DELAY, 5.0)));
 		gth_slideshow_set_automatic (GTH_SLIDESHOW (slideshow), eel_gconf_get_boolean (PREF_SLIDESHOW_AUTOMATIC, TRUE));
 		gth_slideshow_set_wrap_around (GTH_SLIDESHOW (slideshow), eel_gconf_get_boolean (PREF_SLIDESHOW_WRAP_AROUND, FALSE));
-		transition_id = eel_gconf_get_string (PREF_SLIDESHOW_TRANSITION, DEFAULT_TRANSITION);
 	}
 
 	if (g_file_info_get_attribute_status (location->info, "slideshow::playlist") == G_FILE_ATTRIBUTE_STATUS_SET)
@@ -96,6 +127,7 @@ gth_browser_activate_action_view_slideshow (GtkAction  *action,
 
 	_g_object_list_unref (transitions);
 	g_free (transition_id);
+	_g_object_list_unref (filtered_list);
 	_g_object_list_unref (file_list);
 	_gtk_tree_path_list_free (items);
 }
