@@ -207,7 +207,7 @@ static GtkToggleActionEntry gth_browser_action_toggle_entries[] = {
 	  G_CALLBACK (gth_browser_activate_action_view_thumbnails),
 	  TRUE },
 	{ "View_ShowHiddenFiles", NULL,
-	  N_("Show _Hidden Files"), "<control>H",
+	  N_("_Hidden Files"), "<control>H",
 	  N_("Show hidden files and folders"),
 	  G_CALLBACK (gth_browser_activate_action_view_show_hidden_files),
 	  FALSE },
