@@ -167,10 +167,12 @@ gth_cell_renderer_thumbnail_get_size (GtkCellRenderer *cell,
 				      int             *height)
 {
 	GthCellRendererThumbnail *self;
-	int image_width;
-	int image_height;
-	int calc_width;
-  	int calc_height;
+	int   image_width;
+	int   image_height;
+	int   calc_width;
+  	int   calc_height;
+  	int   xpad;
+  	int   ypad;
 
   	self = GTH_CELL_RENDERER_THUMBNAIL (cell);
 
@@ -183,13 +185,15 @@ gth_cell_renderer_thumbnail_get_size (GtkCellRenderer *cell,
   		image_height = 0;
   	}
 
+  	gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+
 	if (self->priv->is_icon || self->priv->fixed_size || (self->priv->thumbnail == NULL) || ((image_width < self->priv->size) && (image_height < self->priv->size))) {
-		calc_width  = (int) (cell->xpad * 2) + (THUMBNAIL_X_BORDER * 2) + self->priv->size;
-		calc_height = (int) (cell->ypad * 2) + (THUMBNAIL_Y_BORDER * 2) + self->priv->size;
+		calc_width  = (int) (xpad * 2) + (THUMBNAIL_X_BORDER * 2) + self->priv->size;
+		calc_height = (int) (ypad * 2) + (THUMBNAIL_Y_BORDER * 2) + self->priv->size;
 	}
 	else {
-		calc_width  = (int) (cell->xpad * 2) + (THUMBNAIL_X_BORDER * 2) + image_width;
-		calc_height = (int) (cell->ypad * 2) + (THUMBNAIL_Y_BORDER * 2) + image_height;
+		calc_width  = (int) (xpad * 2) + (THUMBNAIL_X_BORDER * 2) + image_width;
+		calc_height = (int) (ypad * 2) + (THUMBNAIL_Y_BORDER * 2) + image_height;
 	}
 
 	if (width != NULL)
@@ -199,13 +203,18 @@ gth_cell_renderer_thumbnail_get_size (GtkCellRenderer *cell,
 		*height = calc_height;
 
 	if (cell_area != NULL) {
+	  	float xalign;
+	  	float yalign;
+
+		gtk_cell_renderer_get_alignment (cell, &xalign, &yalign);
+
 		if (x_offset != NULL) {
-			*x_offset = cell->xalign * (cell_area->width - calc_width);
+			*x_offset = xalign * (cell_area->width - calc_width);
 			*x_offset = MAX (*x_offset, 0);
 		}
 
 		if (y_offset != NULL) {
-      			*y_offset = cell->yalign * (cell_area->height - calc_height);
+      			*y_offset = yalign * (cell_area->height - calc_height);
       			*y_offset = MAX (*y_offset, 0);
     		}
   	}
@@ -266,6 +275,29 @@ create_colorized_pixbuf (GdkPixbuf *src,
 }
 
 
+static void
+_cairo_draw_rounded_box (cairo_t *cr,
+			 double   x,
+			 double   y,
+			 double   w,
+			 double   h,
+			 double   r)
+{
+	cairo_move_to (cr, x, y + r);
+	if (r > 0)
+		cairo_arc (cr, x + r, y + r, r, 1.0 * M_PI, 1.5 * M_PI);
+	cairo_rel_line_to (cr, w - (r * 2), 0);
+	if (r > 0)
+		cairo_arc (cr, x + w - r, y + r, r, 1.5 * M_PI, 2.0 * M_PI);
+	cairo_rel_line_to (cr, 0, h - (r * 2));
+	if (r > 0)
+		cairo_arc (cr, x + w - r, y + h - r, r, 0.0 * M_PI, 0.5 * M_PI);
+	cairo_rel_line_to (cr, - (w - (r * 2)), 0);
+	if (r > 0)
+		cairo_arc (cr, x + r, y + h - r, r, 0.5 * M_PI, 1.0 * M_PI);
+}
+
+
 static void
 gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 				    GdkWindow            *window,
@@ -283,8 +315,10 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 	GdkRectangle              image_rect;
 	cairo_t                  *cr;
 	GdkPixbuf                *pixbuf;
+	int                       xpad;
+	int                       ypad;
 	GdkPixbuf                *colorized = NULL;
-	int                       B;
+	int                       border;
 
 	self = GTH_CELL_RENDERER_THUMBNAIL (cell);
 
@@ -298,10 +332,12 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 	if (pixbuf == NULL)
 		return;
 
-	thumb_rect.x += cell_area->x + cell->xpad;
-  	thumb_rect.y += cell_area->y + cell->ypad;
-  	thumb_rect.width  -= cell->xpad * 2;
-  	thumb_rect.height -= cell->ypad * 2;
+	gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+
+	thumb_rect.x += cell_area->x + xpad;
+  	thumb_rect.y += cell_area->y + ypad;
+  	thumb_rect.width  -= xpad * 2;
+  	thumb_rect.height -= ypad * 2;
 
 	if (! gdk_rectangle_intersect (cell_area, &thumb_rect, &draw_rect)
 	    || ! gdk_rectangle_intersect (expose_area, &thumb_rect, &draw_rect))
@@ -319,42 +355,41 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 	style = gtk_widget_get_style (widget);
 
   	if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
-  		state = GTK_WIDGET_HAS_FOCUS (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+  		state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
   	else
   		state = ((flags & GTK_CELL_RENDERER_FOCUSED) == GTK_CELL_RENDERER_FOCUSED) ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
 
 	if (self->priv->is_icon || ((image_rect.width < self->priv->size) && (image_rect.height < self->priv->size))) {
-		int R = 7;
+		GtkStyle *style;
 
+		/* use a gray rounded box for icons or when the original size
+		 * is smaller than the thumbnail size... */
+
+		style = gtk_widget_get_style (widget);
 		if (state == GTK_STATE_NORMAL)
-			gdk_cairo_set_source_color (cr, &widget->style->bg[state]);
+			gdk_cairo_set_source_color (cr, &style->bg[state]);
 		else
-			gdk_cairo_set_source_color (cr, &widget->style->base[state]);
+			gdk_cairo_set_source_color (cr, &style->base[state]);
 
-		cairo_move_to (cr, thumb_rect.x, thumb_rect.y + R);
-		cairo_arc (cr, thumb_rect.x + R, thumb_rect.y + R, R, 1.0 * M_PI, 1.5 * M_PI);
-		cairo_rel_line_to (cr, thumb_rect.width - (R * 2), 0);
-		cairo_arc (cr, thumb_rect.x + thumb_rect.width - R, thumb_rect.y + R, R, 1.5 * M_PI, 2.0 * M_PI);
-		cairo_rel_line_to (cr, 0, thumb_rect.height - (R * 2));
-		cairo_arc (cr, thumb_rect.x + thumb_rect.width - R, thumb_rect.y + thumb_rect.height - R, R, 0.0 * M_PI, 0.5 * M_PI);
-		cairo_rel_line_to (cr, - (thumb_rect.width - (R * 2)), 0);
-		cairo_arc (cr, thumb_rect.x + R, thumb_rect.y + thumb_rect.height - R, R, 0.5 * M_PI, 1.0 * M_PI);
+		_cairo_draw_rounded_box (cr, thumb_rect.x, thumb_rect.y, thumb_rect.width, thumb_rect.height, 7);
 		cairo_close_path (cr);
 		cairo_fill (cr);
 	}
+	else {
+
+		/* ...else draw a frame with a drop-shadow effect */
 
-	if (! self->priv->is_icon && ! ((image_rect.width < self->priv->size) && (image_rect.height < self->priv->size))) {
 		GdkRectangle frame_rect;
 
 		if (state == GTK_STATE_ACTIVE)
 			state = GTK_STATE_SELECTED;
 
 		if (! _g_mime_type_is_image (gth_file_data_get_mime_type (self->priv->file))) {
-			B = 1;
+			border = 1;
 			frame_rect = image_rect;
 		}
 		else {
-			B = 4;
+			border = 4;
 
 			/*frame_rect = thumb_rect;
 			frame_rect.width = self->priv->size;
@@ -365,44 +400,41 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 			frame_rect = image_rect;
 		}
 
-	  	gdk_draw_rectangle (GDK_DRAWABLE (window), style->dark_gc[state], FALSE,
-  				    frame_rect.x - B,
-  				    frame_rect.y - B,
-  				    frame_rect.width + (B * 2) - 1,
-				    frame_rect.height + (B * 2) - 1);
+		cairo_set_line_width (cr, 0.5);
+		cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
+		gdk_cairo_set_source_color (cr, &style->dark[state]);
 
-  		gdk_draw_line (GDK_DRAWABLE (window), style->dark_gc[state],
-  			       frame_rect.x - (B / 2),
-  			       frame_rect.y + frame_rect.height + B,
-  			       frame_rect.x + frame_rect.width + B,
-  			       frame_rect.y + frame_rect.height + B);
-  		gdk_draw_line (GDK_DRAWABLE (window), style->dark_gc[state],
-  			       frame_rect.x + frame_rect.width + B,
-  			       frame_rect.y + frame_rect.height + B,
-  			       frame_rect.x + frame_rect.width + B,
-  			       frame_rect.y - (B / 2));
+		/* the outer frame */
 
-		gdk_draw_line (GDK_DRAWABLE (window), style->mid_gc[state],
-  			       frame_rect.x - (B / 2) + 1,
-  			       frame_rect.y + frame_rect.height + B + 1,
-  			       frame_rect.x + frame_rect.width + B + 1,
-  			       frame_rect.y + frame_rect.height + B + 1);
-  		gdk_draw_line (GDK_DRAWABLE (window), style->mid_gc[state],
-  			       frame_rect.x + frame_rect.width + B + 1,
-  			       frame_rect.y + frame_rect.height + B + 1,
-  			       frame_rect.x + frame_rect.width + B + 1,
-  			       frame_rect.y - (B / 2) + 1);
+		cairo_rectangle (cr,
+				 frame_rect.x - border + 0.5,
+				 frame_rect.y - border + 0.5,
+				 frame_rect.width + (border * 2) - 1,
+				 frame_rect.height + (border * 2) - 1);
 
-  		/*gdk_draw_rectangle (GDK_DRAWABLE (window), style->light_gc[state], TRUE,
-  				    frame_rect.x - (B - 1),
-  				    frame_rect.y - (B - 1),
-  				    frame_rect.width + ((B - 1) * 2),
-  				    frame_rect.height + ((B - 1) * 2));*/
-  		/*gdk_draw_rectangle (GDK_DRAWABLE (window), style->mid_gc[state], FALSE,
-  				    image_rect.x - 1,
-  				    image_rect.y - 1,
-  				    image_rect.width + 1,
-  				    image_rect.height + 1);*/
+		/* the drop shadow */
+
+		cairo_move_to (cr,
+			       frame_rect.x - (border / 2) + 0.5,
+			       frame_rect.y + frame_rect.height + border + 0.5);
+		cairo_line_to (cr,
+			       frame_rect.x + frame_rect.width + border + 0.5,
+			       frame_rect.y + frame_rect.height + border + 0.5);
+		cairo_line_to (cr,
+			       frame_rect.x + frame_rect.width + border + 0.5,
+			       frame_rect.y - (border / 2) + 0.5);
+
+		cairo_move_to (cr,
+			       frame_rect.x - (border / 2) + 0.5,
+			       frame_rect.y + frame_rect.height + border + 1.5);
+		cairo_line_to (cr,
+			       frame_rect.x + frame_rect.width + border + 1.5,
+			       frame_rect.y + frame_rect.height + border + 1.5);
+		cairo_line_to (cr,
+			       frame_rect.x + frame_rect.width + border + 1.5,
+			       frame_rect.y - (border / 2) + 0.5);
+
+		cairo_stroke (cr);
 	}
 
   	if (! self->priv->checked || ((flags & (GTK_CELL_RENDERER_SELECTED | GTK_CELL_RENDERER_PRELIT)) != 0)) {
@@ -410,33 +442,12 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 		pixbuf = colorized;
 	}
 
-  	gdk_draw_pixbuf (GDK_DRAWABLE (window),
-  			 NULL,
-  			 pixbuf,
-  			 0, 0,
-  			 image_rect.x,
-  			 image_rect.y,
-  			 image_rect.width,
-  			 image_rect.height,
-  			 GDK_RGB_DITHER_NORMAL,
-  			 0, 0);
-
-  	/*if (! self->priv->checked) {
-  		cairo_set_source_rgba (cr, 1.0, 0.0, 0.0, 0.5);
-		cairo_set_line_width (cr, 6.0);
-
-		cairo_move_to (cr, thumb_rect.x, thumb_rect.y);
-		cairo_line_to (cr, thumb_rect.x + thumb_rect.width, thumb_rect.y + thumb_rect.height);
-		cairo_stroke (cr);
-		cairo_move_to (cr, thumb_rect.x + thumb_rect.width, thumb_rect.y);
-		cairo_line_to (cr, thumb_rect.x, thumb_rect.y + thumb_rect.height);
-		cairo_stroke (cr);
-  	}*/
+  	gdk_cairo_set_source_pixbuf (cr, pixbuf, image_rect.x, image_rect.y);
+  	cairo_rectangle (cr, image_rect.x, image_rect.y, image_rect.width, image_rect.height);
+  	cairo_fill (cr);
 
+  	_g_object_unref (colorized);
   	cairo_destroy (cr);
-
-  	if (colorized != NULL)
-		g_object_unref (colorized);
 }
 
 
