@@ -36,8 +36,7 @@
 #include "gtk-utils.h"
 
 #define DEFAULT_THUMBNAIL_SIZE 112
-#define N_THUMBS_PER_NOTIFICATION 15
-#define UPDATE_THUMBNAILS_TIMEOUT 250
+#define UPDATE_THUMBNAILS_TIMEOUT 200
 #define N_LOOKAHEAD 50
 #define EMPTY (N_("(Empty)"))
 #define THUMBNAIL_BORDER (8 * 2)
@@ -318,23 +317,41 @@ gth_file_list_init (GthFileList *file_list)
 static void _gth_file_list_update_next_thumb (GthFileList *file_list);
 
 
+static void
+flash_queue (GthFileList *file_list)
+{
+	if (file_list->priv->dirty) {
+		GthFileStore *file_store;
+
+		file_store = (GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
+		gth_file_store_exec_set (file_store);
+		file_list->priv->dirty = FALSE;
+	}
+
+	if (file_list->priv->dirty_event != 0) {
+		g_source_remove (file_list->priv->dirty_event);
+		file_list->priv->dirty_event = 0;
+	}
+}
+
+
 static gboolean
 flash_queue_cb (gpointer data)
 {
-	GthFileList *file_list = data;
-	GthFileStore *file_store;
-
-	file_store = (GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
-	gth_file_store_exec_set (file_store);
-
-	g_source_remove (file_list->priv->dirty_event);
-	file_list->priv->dirty_event = 0;
-	file_list->priv->dirty = FALSE;
-
+	flash_queue ((GthFileList *) data);
 	return FALSE;
 }
 
 
+static void
+queue_flash_updates (GthFileList *file_list)
+{
+	file_list->priv->dirty = TRUE;
+	if (file_list->priv->dirty_event == 0)
+		file_list->priv->dirty_event = g_timeout_add (UPDATE_THUMBNAILS_TIMEOUT, flash_queue_cb, file_list);
+}
+
+
 static void
 update_thumb_in_file_view (GthFileList *file_list)
 {
@@ -353,9 +370,7 @@ update_thumb_in_file_view (GthFileList *file_list)
 						  GTH_FILE_STORE_THUMBNAIL_COLUMN, pixbuf,
 						  GTH_FILE_STORE_IS_ICON_COLUMN, FALSE,
 						  -1);
-			file_list->priv->dirty = TRUE;
-			if (file_list->priv->dirty_event == 0)
-				file_list->priv->dirty_event = g_timeout_add (UPDATE_THUMBNAILS_TIMEOUT, flash_queue_cb, file_list);
+			queue_flash_updates (file_list);
 		}
 	}
 }
@@ -382,6 +397,7 @@ set_mime_type_icon (GthFileList *file_list,
 				  GTH_FILE_STORE_THUMBNAIL_COLUMN, pixbuf,
 				  GTH_FILE_STORE_IS_ICON_COLUMN, TRUE,
 				  -1);
+	queue_flash_updates (file_list);
 
 	_g_object_unref (pixbuf);
 }
@@ -514,19 +530,47 @@ checkbox_toggled_cb (GtkCellRendererToggle *cell_renderer,
 }
 
 
+static void
+_gth_file_list_set_type (GthFileList     *file_list,
+		 	 GthFileListType  list_type)
+{
+	file_list->priv->type = list_type;
+
+	if ((file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_NO_SELECTION))
+		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_NONE);
+	else if ((file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR))
+		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_SINGLE);
+	else
+		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_MULTIPLE);
+
+	g_object_set (file_list->priv->checkbox_renderer,
+		      "visible", ((file_list->priv->type == GTH_FILE_LIST_TYPE_BROWSER) || (file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR)),
+		      NULL);
+
+	g_object_set (file_list->priv->thumbnail_renderer,
+		      "fixed_size", (file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR),
+		      NULL);
+
+	if (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR)
+		gtk_widget_set_size_request (GTK_WIDGET (file_list->priv->view),
+					     file_list->priv->thumb_size + (THUMBNAIL_BORDER * 2),
+					     -1);
+	else if (file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR)
+		gtk_widget_set_size_request (GTK_WIDGET (file_list->priv->view),
+					     -1,
+					     file_list->priv->thumb_size + (THUMBNAIL_BORDER * 2));
+}
+
+
 static void
 gth_file_list_construct (GthFileList     *file_list,
-			 GthFileListType  list_type)
+			 GthFileListType  list_type,
+			 gboolean         enable_drag_drop)
 {
 	GtkWidget       *scrolled;
 	GtkWidget       *viewport;
 	GtkCellRenderer *renderer;
 	GthFileStore    *model;
-	GtkTargetList   *target_list;
-	GtkTargetEntry  *targets;
-	int              n_targets;
-
-	file_list->priv->type = list_type;
 
 	/* thumbnail loader */
 
@@ -575,30 +619,29 @@ gth_file_list_construct (GthFileList     *file_list,
 	file_list->priv->view = gth_icon_view_new_with_model (GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
-	if ((file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_NO_SELECTION))
-		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_NONE);
-	else if (file_list->priv->type == GTH_FILE_LIST_TYPE_THUMBNAIL)
-		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_SINGLE);
-	else
-		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_MULTIPLE);
+	if (enable_drag_drop) {
+		GtkTargetList  *target_list;
+		GtkTargetEntry *targets;
+		int             n_targets;
 
-	target_list = gtk_target_list_new (NULL, 0);
-	gtk_target_list_add_uri_targets (target_list, 0);
-	gtk_target_list_add_text_targets (target_list, 0);
-	targets = gtk_target_table_new_from_list (target_list, &n_targets);
-	gth_file_view_enable_drag_source (GTH_FILE_VIEW (file_list->priv->view),
-					  GDK_BUTTON1_MASK,
-					  targets,
-					  n_targets,
-					  GDK_ACTION_MOVE | GDK_ACTION_COPY);
+		target_list = gtk_target_list_new (NULL, 0);
+		gtk_target_list_add_uri_targets (target_list, 0);
+		gtk_target_list_add_text_targets (target_list, 0);
+		targets = gtk_target_table_new_from_list (target_list, &n_targets);
+		gth_file_view_enable_drag_source (GTH_FILE_VIEW (file_list->priv->view),
+						  GDK_BUTTON1_MASK,
+						  targets,
+						  n_targets,
+						  GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
-	gtk_target_list_unref (target_list);
-	gtk_target_table_free (targets, n_targets);
+		gtk_target_list_unref (target_list);
+		gtk_target_table_free (targets, n_targets);
 
-	g_signal_connect (G_OBJECT (file_list->priv->view),
-			  "drag-data-get",
-			  G_CALLBACK (file_view_drag_data_get_cb),
-			  file_list);
+		g_signal_connect (G_OBJECT (file_list->priv->view),
+				  "drag-data-get",
+				  G_CALLBACK (file_view_drag_data_get_cb),
+				  file_list);
+	}
 
 	/* checkbox */
 
@@ -608,9 +651,6 @@ gth_file_list_construct (GthFileList     *file_list,
 					renderer,
 					"active", GTH_FILE_STORE_CHECKED_COLUMN,
 					NULL);
-	g_object_set (file_list->priv->checkbox_renderer,
-		      "visible", ((file_list->priv->type == GTH_FILE_LIST_TYPE_BROWSER) || (file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR)),
-		      NULL);
 	g_signal_connect (file_list->priv->checkbox_renderer,
 			  "toggled",
 			  G_CALLBACK (checkbox_toggled_cb),
@@ -622,7 +662,6 @@ gth_file_list_construct (GthFileList     *file_list,
 	g_object_set (renderer,
 		      "size", file_list->priv->thumb_size,
 		      "yalign", 1.0,
-		      "fixed_size", (file_list->priv->type == GTH_FILE_LIST_TYPE_THUMBNAIL),
 		      NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (file_list->priv->view), renderer, FALSE);
 
@@ -661,6 +700,8 @@ gth_file_list_construct (GthFileList     *file_list,
 					"text", GTH_FILE_STORE_METADATA_COLUMN,
 					NULL);
 
+	_gth_file_list_set_type (file_list, list_type);
+
 	/* pack the widgets together */
 
 	gtk_widget_show (file_list->priv->view);
@@ -711,17 +752,26 @@ gth_file_list_get_type (void)
 
 
 GtkWidget*
-gth_file_list_new (GthFileListType list_type)
+gth_file_list_new (GthFileListType list_type,
+		   gboolean        enable_drag_drop)
 {
 	GtkWidget *widget;
 
 	widget = GTK_WIDGET (g_object_new (GTH_TYPE_FILE_LIST, NULL));
-	gth_file_list_construct (GTH_FILE_LIST (widget), list_type);
+	gth_file_list_construct (GTH_FILE_LIST (widget), list_type, enable_drag_drop);
 
 	return widget;
 }
 
 
+void
+gth_file_list_set_type (GthFileList     *file_list,
+			GthFileListType  list_type)
+{
+	_gth_file_list_set_type (file_list, list_type);
+}
+
+
 static void
 _gth_file_list_thumb_cleanup (GthFileList *file_list)
 {
@@ -1218,7 +1268,11 @@ gth_file_list_set_thumb_size (GthFileList *file_list,
 		      "wrap-width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
 		      NULL);
 
-	if (file_list->priv->type == GTH_FILE_LIST_TYPE_THUMBNAIL)
+	if (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR)
+		gtk_widget_set_size_request (GTK_WIDGET (file_list->priv->view),
+					     file_list->priv->thumb_size + (THUMBNAIL_BORDER * 2),
+					     -1);
+	else if (file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR)
 		gtk_widget_set_size_request (GTK_WIDGET (file_list->priv->view),
 					     -1,
 					     file_list->priv->thumb_size + (THUMBNAIL_BORDER * 2));
@@ -1291,12 +1345,7 @@ gth_file_list_get_vadjustment (GthFileList *file_list)
 static void
 _gth_file_list_thumbs_completed (GthFileList *file_list)
 {
-	GthFileStore *file_store;
-
-	file_store = (GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
-	if (file_list->priv->n_thumb >= 0)
-		gth_file_store_exec_set (file_store);
-
+	flash_queue (file_list);
 	_gth_file_list_done (file_list);
 }
 
@@ -1322,6 +1371,7 @@ set_loading_icon (GthFileList *file_list,
 				  GTH_FILE_STORE_THUMBNAIL_COLUMN, pixbuf,
 				  GTH_FILE_STORE_IS_ICON_COLUMN, TRUE,
 				  -1);
+	queue_flash_updates (file_list);
 
 	_g_object_unref (pixbuf);
 	g_object_unref (icon);
