@@ -345,12 +345,47 @@ row_expanded_cb (GtkTreeView  *tree_view,
 }
 
 
+static gboolean
+popup_menu_cb (GtkWidget *widget,
+	       gpointer   user_data)
+{
+	GthFolderTree *folder_tree = user_data;
+	GtkTreeStore  *tree_store = folder_tree->priv->tree_store;
+	GthFileData   *file_data = NULL;
+	GtkTreeIter    iter;
+
+	if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (folder_tree)), NULL, &iter)) {
+		EntryType entry_type;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (tree_store),
+				    &iter,
+				    COLUMN_TYPE, &entry_type,
+				    COLUMN_FILE_DATA, &file_data,
+				    -1);
+		if (entry_type != ENTRY_TYPE_FILE) {
+			_g_object_unref (file_data);
+			return FALSE;
+		}
+	}
+
+	g_signal_emit (folder_tree,
+		       gth_folder_tree_signals[FOLDER_POPUP],
+		       0,
+		       file_data,
+		       gtk_get_current_event_time ());
+
+	_g_object_unref (file_data);
+
+	return TRUE;
+}
+
+
 static int
 button_press_cb (GtkWidget      *widget,
 		 GdkEventButton *event,
-		 gpointer        data)
+		 gpointer        user_data)
 {
-	GthFolderTree     *folder_tree = data;
+	GthFolderTree     *folder_tree = user_data;
 	GtkTreeStore      *tree_store = folder_tree->priv->tree_store;
 	GtkTreePath       *path;
 	GtkTreeIter        iter;
@@ -972,6 +1007,10 @@ gth_folder_tree_construct (GthFolderTree *folder_tree)
 
 	/**/
 
+	g_signal_connect (folder_tree,
+			  "popup-menu",
+			  G_CALLBACK (popup_menu_cb),
+			  folder_tree);
 	g_signal_connect (folder_tree,
 			  "button-press-event",
 			  G_CALLBACK (button_press_cb),
@@ -1474,25 +1513,21 @@ gth_folder_tree_start_editing (GthFolderTree *folder_tree,
 {
 	GtkTreeIter        iter;
 	GtkTreePath       *tree_path;
-	char              *path;
 	GtkTreeViewColumn *tree_column;
 
 	if (! gth_folder_tree_get_iter (folder_tree, file, &iter, NULL))
 		return;
 
 	tree_path = gtk_tree_model_get_path (GTK_TREE_MODEL (folder_tree->priv->tree_store), &iter);
-	path = gtk_tree_path_to_string (tree_path);
-
-	tree_column = gtk_tree_view_get_column (GTK_TREE_VIEW (folder_tree), 0);
 	gtk_tree_view_expand_to_path (GTK_TREE_VIEW (folder_tree), tree_path);
 	gtk_tree_view_collapse_row (GTK_TREE_VIEW (folder_tree), tree_path);
 	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (folder_tree), tree_path, NULL, TRUE, 0.5, 0.0);
+	tree_column = gtk_tree_view_get_column (GTK_TREE_VIEW (folder_tree), 0);
 	gtk_tree_view_set_cursor (GTK_TREE_VIEW (folder_tree),
 				  tree_path,
 				  tree_column,
 				  TRUE);
 
-	g_free (path);
 	gtk_tree_path_free (tree_path);
 }
 
