@@ -97,17 +97,26 @@ gth_image_dragger_unrealize (GthImageViewerTool *base)
 
 
 static void
-gth_image_dragger_size_allocate (GthImageViewerTool  *base,
-				 GtkAllocation       *allocation)
+gth_image_dragger_size_allocate (GthImageViewerTool *base,
+				 GtkAllocation      *allocation)
 {
 	GthImageDragger *self;
 	GthImageViewer  *viewer;
+	double           h_page_size;
+	double           v_page_size;
+	double           h_upper;
+	double           v_upper;
 
 	self = (GthImageDragger *) base;
 	viewer = (GthImageViewer *) self->priv->viewer;
 
-	self->priv->draggable = (viewer->hadj->upper > viewer->hadj->page_size) || (viewer->vadj->upper > viewer->vadj->page_size);
-	if (GTK_WIDGET_REALIZED (viewer))
+	h_page_size = gtk_adjustment_get_page_size (viewer->hadj);
+	v_page_size = gtk_adjustment_get_page_size (viewer->vadj);
+	h_upper = gtk_adjustment_get_upper (viewer->hadj);
+	v_upper = gtk_adjustment_get_upper (viewer->vadj);
+
+	self->priv->draggable = (h_page_size > 0) && (v_page_size > 0) && ((h_upper > h_page_size) || (v_upper > v_page_size));
+	if (gtk_widget_get_realized (GTK_WIDGET (viewer)))
 		_gth_image_dragger_update_cursor (self);
 }
 
@@ -128,12 +137,12 @@ gth_image_dragger_unmap (GthImageViewerTool *base)
 
 static void
 gth_image_dragger_expose (GthImageViewerTool *self,
-			  GdkRectangle       *event_area)
+			  GdkEventExpose     *event,
+		          cairo_t            *cr)
 {
 	GthImageDragger *dragger;
 	GthImageViewer  *viewer;
 	GdkInterpType    interp_type;
-	GdkRectangle     paint_area;
 
 	dragger = (GthImageDragger *) self;
 	viewer = dragger->priv->viewer;
@@ -149,16 +158,14 @@ gth_image_dragger_expose (GthImageViewerTool *self,
 	if (gth_image_viewer_get_zoom (viewer) == 1.0)
 		interp_type = GDK_INTERP_NEAREST;
 
-	if (gdk_rectangle_intersect (&viewer->image_area, event_area, &paint_area))
-		gth_image_viewer_paint (viewer,
-					gth_image_viewer_get_current_pixbuf (viewer),
-					viewer->x_offset + paint_area.x - viewer->image_area.x,
-					viewer->y_offset + paint_area.y - viewer->image_area.y,
-					paint_area.x,
-					paint_area.y,
-					paint_area.width,
-					paint_area.height,
-					interp_type);
+	gth_image_viewer_paint_region (viewer,
+				       cr,
+				       gth_image_viewer_get_current_pixbuf (viewer),
+				       viewer->x_offset - viewer->image_area.x,
+				       viewer->y_offset - viewer->image_area.y,
+				       &viewer->image_area,
+				       event->region,
+				       interp_type);
 }
 
 
@@ -181,8 +188,8 @@ gth_image_dragger_button_press (GthImageViewerTool *self,
 		GdkCursor *cursor;
 		int        retval;
 
-		cursor = gth_cursor_get (widget->window, GTH_CURSOR_HAND_CLOSED);
-		retval = gdk_pointer_grab (widget->window,
+		cursor = gth_cursor_get (gtk_widget_get_window (widget), GTH_CURSOR_HAND_CLOSED);
+		retval = gdk_pointer_grab (gtk_widget_get_window (widget),
 					   FALSE,
 					   (GDK_POINTER_MOTION_MASK
 					    | GDK_POINTER_MOTION_HINT_MASK
