@@ -35,6 +35,7 @@
 #define STEP_INCREMENT 20.0  /* scroll increment. */
 #define SCROLL_TIMEOUT 30    /* autoscroll timeout in milliseconds */
 #define GOLDEN_RATIO   1.6180339887
+#define GOLDER_RATIO_FACTOR (GOLDEN_RATIO / (1.0 + 2.0 * GOLDEN_RATIO))
 
 
 typedef struct {
@@ -204,9 +205,10 @@ struct _GthImageSelectorPrivate {
 	gboolean         mask_visible;
 	GthGridType      grid_type;
 	gboolean         active;
+	gboolean         bind_dimensions;
+	int              bind_factor;
 
 	GdkRectangle     drag_start_selection_area;
-	GdkGC           *selection_gc;
 	GdkRectangle     selection_area;
 	GdkRectangle     selection;
 
@@ -349,7 +351,7 @@ update_event_areas (GthImageSelector *self)
 	EventArea *event_area;
 	int        x, y, width, height;
 
-	if (! GTK_WIDGET_REALIZED (self->priv->viewer))
+	if (! gtk_widget_get_realized (GTK_WIDGET (self->priv->viewer)))
 		return;
 
 	x = self->priv->selection_area.x - 1;
@@ -417,7 +419,7 @@ static void
 queue_draw (GthImageSelector *self,
 	    GdkRectangle      area)
 {
-	if (! GTK_WIDGET_REALIZED (self->priv->viewer))
+	if (! gtk_widget_get_realized (GTK_WIDGET (self->priv->viewer)))
 		return;
 
 	gtk_widget_queue_draw_area (GTK_WIDGET (self->priv->viewer),
@@ -512,15 +514,6 @@ gth_image_selector_realize (GthImageViewerTool *base)
 
 	widget = (GtkWidget *) self->priv->viewer;
 
-	self->priv->selection_gc = gdk_gc_new (widget->window);
-	gdk_gc_copy (self->priv->selection_gc, widget->style->white_gc);
-	gdk_gc_set_line_attributes (self->priv->selection_gc,
-				    1,
-				    GDK_LINE_ON_OFF_DASH /*GDK_LINE_SOLID*/,
-				    GDK_CAP_BUTT,
-				    GDK_JOIN_MITER);
-	gdk_gc_set_function (self->priv->selection_gc, GDK_INVERT);
-
 	if (self->priv->type == GTH_SELECTOR_TYPE_REGION)
 		self->priv->default_cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_CROSSHAIR /*GDK_LEFT_PTR*/);
 	else if (self->priv->type == GTH_SELECTOR_TYPE_POINT)
@@ -552,11 +545,6 @@ gth_image_selector_unrealize (GthImageViewerTool *base)
 		self->priv->default_cursor = NULL;
 	}
 
-	if (self->priv->selection_gc != NULL) {
-		g_object_unref (self->priv->selection_gc);
-		self->priv->selection_gc = NULL;
-	}
-
 	free_event_area_list (self);
 }
 
@@ -586,77 +574,73 @@ gth_image_selector_unmap (GthImageViewerTool *base)
 }
 
 
-G_GNUC_UNUSED
-static void
-print_rectangle (const char   *name,
-		 GdkRectangle *r)
-{
-	g_print ("%s ==> (%d,%d) [%d,%d]\n", name, r->x, r->y, r->width, r->height);
-}
-
-
 static void
 paint_background (GthImageSelector *self,
-		  GdkRectangle     *event_area)
+		  GdkEventExpose   *event,
+		  cairo_t          *cr)
 {
-	GdkRectangle paint_area;
-
-	if (! gdk_rectangle_intersect (&self->priv->viewer->image_area, event_area, &paint_area))
-		return;
-
-	gth_image_viewer_paint (self->priv->viewer,
-				self->priv->background,
-				self->priv->viewer->x_offset + paint_area.x - self->priv->viewer->image_area.x,
-				self->priv->viewer->y_offset + paint_area.y - self->priv->viewer->image_area.y,
-				paint_area.x,
-				paint_area.y,
-				paint_area.width,
-				paint_area.height,
-				GDK_INTERP_TILES);
+	gth_image_viewer_paint_region (self->priv->viewer,
+				       cr,
+				       self->priv->background,
+				       self->priv->viewer->x_offset - self->priv->viewer->image_area.x,
+				       self->priv->viewer->y_offset - self->priv->viewer->image_area.y,
+				       &self->priv->viewer->image_area,
+				       event->region,
+				       GDK_INTERP_TILES);
 }
 
 
 static void
 paint_selection (GthImageSelector *self,
-		 GdkRectangle     *event_area)
+		 GdkEventExpose   *event,
+		 cairo_t          *cr)
 {
-	GdkRectangle selection_area, paint_area;
+	GdkRectangle selection_area;
 
 	selection_area = self->priv->selection_area;
 	selection_area.x += self->priv->viewer->image_area.x - self->priv->viewer->x_offset;
 	selection_area.y += self->priv->viewer->image_area.y - self->priv->viewer->y_offset;
 
-	if (! gdk_rectangle_intersect (&selection_area, event_area, &paint_area))
-		return;
+	gth_image_viewer_paint_region (self->priv->viewer,
+				       cr,
+				       self->priv->pixbuf,
+				       self->priv->viewer->x_offset - self->priv->viewer->image_area.x,
+				       self->priv->viewer->y_offset - self->priv->viewer->image_area.y,
+				       &selection_area,
+				       event->region,
+				       GDK_INTERP_TILES);
 
-	gth_image_viewer_paint (self->priv->viewer,
-				self->priv->pixbuf,
-				self->priv->viewer->x_offset + paint_area.x - self->priv->viewer->image_area.x,
-				self->priv->viewer->y_offset + paint_area.y - self->priv->viewer->image_area.y,
-				paint_area.x,
-				paint_area.y,
-				paint_area.width,
-				paint_area.height,
-				GDK_INTERP_TILES);
+	cairo_save (cr);
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 9, 2)
+	cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+#else
+	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+#endif
+
+
+	gdk_cairo_region (cr, event->region);
+	gdk_cairo_rectangle (cr, &selection_area);
+	cairo_clip (cr);
 
 	if (self->priv->grid_type != GTH_GRID_NONE) {
 		int grid_x0, grid_x1, grid_x2, grid_x3;
 	        int grid_y0, grid_y1, grid_y2, grid_y3;
 		int x_delta, y_delta;
 
-		grid_x0 = paint_area.x;
-		grid_x3 = paint_area.x + paint_area.width;
+		grid_x0 = selection_area.x;
+		grid_x3 = selection_area.x + selection_area.width;
 
-                grid_y0 = paint_area.y;
-                grid_y3 = paint_area.y + paint_area.height;
+                grid_y0 = selection_area.y;
+                grid_y3 = selection_area.y + selection_area.height;
 
 		if (self->priv->grid_type == GTH_GRID_THIRDS) {
-			x_delta = paint_area.width / 3;
-			y_delta = paint_area.height /3;
+			x_delta = selection_area.width / 3;
+			y_delta = selection_area.height /3;
 		}
 		else if (self->priv->grid_type == GTH_GRID_GOLDEN) {
-			x_delta = paint_area.width * (GOLDEN_RATIO / (1.0 + 2.0 * GOLDEN_RATIO));
-			y_delta = paint_area.height * (GOLDEN_RATIO / (1.0 + 2.0 * GOLDEN_RATIO));
+			x_delta = selection_area.width * GOLDER_RATIO_FACTOR;
+			y_delta = selection_area.height * GOLDER_RATIO_FACTOR;
 		}
 
 		grid_x1 = grid_x0 + x_delta;
@@ -664,79 +648,62 @@ paint_selection (GthImageSelector *self,
                 grid_y1 = grid_y0 + y_delta;
                 grid_y2 = grid_y3 - y_delta;
 
-		gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
-			       self->priv->selection_gc,
-			       grid_x1, grid_y0,
-			       grid_x1, grid_y3);
-	        gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
-       		               self->priv->selection_gc,
- 			       grid_x2, grid_y0,
-			       grid_x2, grid_y3);
-        	gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
-       	        	       self->priv->selection_gc,
-                               grid_x0, grid_y1,
-                               grid_x3, grid_y1);
-	        gdk_draw_line (GTK_WIDGET (self->priv->viewer)->window,
-       		               self->priv->selection_gc,
-                               grid_x0, grid_y2,
-                               grid_x3, grid_y2);
+		cairo_move_to (cr, grid_x1 + 0.5, grid_y0 + 0.5);
+		cairo_line_to (cr, grid_x1 + 0.5, grid_y3 + 0.5);
+
+		cairo_move_to (cr, grid_x2 + 0.5, grid_y0 + 0.5);
+		cairo_line_to (cr, grid_x2 + 0.5, grid_y3 + 0.5);
+
+		cairo_move_to (cr, grid_x0 + 0.5, grid_y1 + 0.5);
+		cairo_line_to (cr, grid_x3 + 0.5, grid_y1 + 0.5);
+
+		cairo_move_to (cr, grid_x0 + 0.5, grid_y2 + 0.5);
+		cairo_line_to (cr, grid_x3 + 0.5, grid_y2 + 0.5);
 	}
+
+	cairo_rectangle (cr,
+			 selection_area.x + 0.5,
+			 selection_area.y + 0.5,
+			 selection_area.width,
+			 selection_area.height);
+	cairo_stroke (cr);
+
+	cairo_restore (cr);
 }
 
 
 static void
 paint_image (GthImageSelector *self,
-	     GdkRectangle     *event_area)
+	     GdkEventExpose   *event,
+	     cairo_t          *cr)
 {
-	GdkRectangle paint_area;
-
-	if (! gdk_rectangle_intersect (&self->priv->viewer->image_area, event_area, &paint_area))
-		return;
-
-	gth_image_viewer_paint (self->priv->viewer,
-				self->priv->pixbuf,
-				self->priv->viewer->x_offset + paint_area.x - self->priv->viewer->image_area.x,
-				self->priv->viewer->y_offset + paint_area.y - self->priv->viewer->image_area.y,
-				paint_area.x,
-				paint_area.y,
-				paint_area.width,
-				paint_area.height,
-				GDK_INTERP_TILES);
+	gth_image_viewer_paint_region (self->priv->viewer,
+				       cr,
+				       self->priv->pixbuf,
+				       self->priv->viewer->x_offset - self->priv->viewer->image_area.x,
+				       self->priv->viewer->y_offset - self->priv->viewer->image_area.y,
+				       &self->priv->viewer->image_area,
+				       event->region,
+				       GDK_INTERP_TILES);
 }
 
 
 static void
 gth_image_selector_expose (GthImageViewerTool *base,
-			   GdkRectangle       *event_area)
+			   GdkEventExpose     *event,
+			   cairo_t            *cr)
 {
 	GthImageSelector *self = GTH_IMAGE_SELECTOR (base);
 
 	if (self->priv->pixbuf == NULL)
 		return;
 
-	if (! self->priv->mask_visible) {
-		paint_image (self, event_area);
-		return;
-	}
-
-	paint_background (self, event_area);
-	paint_selection (self, event_area);
-
-	if (GTK_WIDGET_HAS_FOCUS (self->priv->viewer)) {
-		GdkRectangle area;
-
-		area = self->priv->selection_area;
-		area.x += self->priv->viewer->image_area.x - self->priv->viewer->x_offset;
-		area.y += self->priv->viewer->image_area.y - self->priv->viewer->y_offset;
-
-		gdk_draw_rectangle (GTK_WIDGET (self->priv->viewer)->window,
-				    self->priv->selection_gc,
-				    FALSE,
-				    area.x,
-				    area.y,
-				    area.width,
-				    area.height);
+	if (self->priv->mask_visible) {
+		paint_background (self, event, cr);
+		paint_selection (self, event, cr);
 	}
+	else
+		paint_image (self, event, cr);
 }
 
 
@@ -876,6 +843,24 @@ get_semiplane_no (int x1,
 }
 
 
+static int
+bind_dimension (int dimension,
+		int factor)
+{
+	int d;
+	int d_next;
+
+	d = (dimension / factor) * factor;
+	d_next = d + factor;
+	if (dimension - d <= d_next - dimension)
+		dimension = d;
+	else
+		dimension = d_next;
+
+	return dimension;
+}
+
+
 static void
 check_and_set_new_selection (GthImageSelector *self,
 			     GdkRectangle      new_selection)
@@ -883,7 +868,13 @@ check_and_set_new_selection (GthImageSelector *self,
 	new_selection.width = MAX (0, new_selection.width);
 	new_selection.height = MAX (0, new_selection.height);
 
-	if (((self->priv->current_area == NULL) || (self->priv->current_area->id != C_SELECTION_AREA))
+	if (self->priv->bind_dimensions && (self->priv->bind_factor > 1)) {
+		new_selection.width = bind_dimension (new_selection.width, self->priv->bind_factor);
+		new_selection.height = bind_dimension (new_selection.height, self->priv->bind_factor);
+	}
+
+	if (((self->priv->current_area == NULL)
+	     || (self->priv->current_area->id != C_SELECTION_AREA))
 	    && self->priv->use_ratio)
 	{
 		if (rectangle_in_rectangle (new_selection, self->priv->pixbuf_area))
@@ -1147,20 +1138,24 @@ static gboolean
 autoscroll_cb (gpointer data)
 {
 	GthImageSelector *self = GTH_IMAGE_SELECTOR (data);
-	double            max_value;
 	double            value;
+	double            max_value;
 
 	GDK_THREADS_ENTER ();
 
-	max_value = self->priv->viewer->hadj->upper - self->priv->viewer->hadj->page_size;
-	value = self->priv->viewer->hadj->value + self->priv->x_value_diff;
+	/* drag x */
+
+	value = gtk_adjustment_get_value (self->priv->viewer->hadj) + self->priv->x_value_diff;
+	max_value = gtk_adjustment_get_upper (self->priv->viewer->hadj) - gtk_adjustment_get_page_size (self->priv->viewer->hadj);
 	if (value > max_value)
 		value = max_value;
 	gtk_adjustment_set_value (self->priv->viewer->hadj, value);
 	self->priv->viewer->drag_x = self->priv->viewer->drag_x + self->priv->x_value_diff;
 
-	max_value = self->priv->viewer->vadj->upper - self->priv->viewer->vadj->page_size;
-	value = self->priv->viewer->vadj->value + self->priv->y_value_diff;
+	/* drag y */
+
+	value = gtk_adjustment_get_value (self->priv->viewer->vadj) + self->priv->y_value_diff;
+	max_value = gtk_adjustment_get_upper (self->priv->viewer->vadj) - gtk_adjustment_get_page_size (self->priv->viewer->vadj);
 	if (value > max_value)
 		value = max_value;
 	gtk_adjustment_set_value (self->priv->viewer->vadj, value);
@@ -1182,6 +1177,7 @@ gth_image_selector_motion_notify (GthImageViewerTool *base,
 	GthImageSelector *self = GTH_IMAGE_SELECTOR (base);
 	GtkWidget        *widget;
 	int               x, y;
+	GtkAllocation     allocation;
 	int               absolute_x, absolute_y;
 
 	widget = GTK_WIDGET (self->priv->viewer);
@@ -1206,7 +1202,7 @@ gth_image_selector_motion_notify (GthImageViewerTool *base,
 	{
 		int retval;
 
-		retval = gdk_pointer_grab (widget->window,
+		retval = gdk_pointer_grab (gtk_widget_get_window (widget),
 					   FALSE,
 					   (GDK_POINTER_MOTION_MASK
 					    | GDK_POINTER_MOTION_HINT_MASK
@@ -1232,11 +1228,11 @@ gth_image_selector_motion_notify (GthImageViewerTool *base,
 	/* If we are out of bounds, schedule a timeout that will do
 	 * the scrolling */
 
+	gtk_widget_get_allocation (widget, &allocation);
 	absolute_x = event->x;
 	absolute_y = event->y;
-
-	if ((absolute_y < 0) || (absolute_y > widget->allocation.height)
-	    || (absolute_x < 0) || (absolute_x > widget->allocation.width))
+	if ((absolute_y < 0) || (absolute_y > allocation.height)
+	    || (absolute_x < 0) || (absolute_x > allocation.width))
 	{
 
 		/* Make the steppings be relative to the mouse
@@ -1246,16 +1242,16 @@ gth_image_selector_motion_notify (GthImageViewerTool *base,
 		 */
 		if (absolute_x < 0)
 			self->priv->x_value_diff = absolute_x;
-		else if (absolute_x > widget->allocation.width)
-			self->priv->x_value_diff = absolute_x - widget->allocation.width;
+		else if (absolute_x > allocation.width)
+			self->priv->x_value_diff = absolute_x - allocation.width;
 		else
 			self->priv->x_value_diff = 0.0;
 		self->priv->x_value_diff /= 2;
 
 		if (absolute_y < 0)
 			self->priv->y_value_diff = absolute_y;
-		else if (absolute_y > widget->allocation.height)
-			self->priv->y_value_diff = absolute_y - widget->allocation.height;
+		else if (absolute_y > allocation.height)
+			self->priv->y_value_diff = absolute_y -allocation.height;
 		else
 			self->priv->y_value_diff = 0.0;
 		self->priv->y_value_diff /= 2;
@@ -1328,6 +1324,8 @@ gth_image_selector_instance_init (GthImageSelector *self)
 	self->priv->ratio = 1.0;
 	self->priv->mask_visible = TRUE;
 	self->priv->grid_type = GTH_GRID_NONE;
+	self->priv->bind_dimensions = FALSE;
+	self->priv->bind_factor = 1;
 }
 
 
@@ -1635,3 +1633,13 @@ gth_image_selector_get_grid_type (GthImageSelector *self)
 {
         return self->priv->grid_type;
 }
+
+
+void
+gth_image_selector_bind_dimensions (GthImageSelector *self,
+				    gboolean          bind,
+				    int               factor)
+{
+	self->priv->bind_dimensions = bind;
+	self->priv->bind_factor = factor;
+}
