@@ -40,7 +40,6 @@
 #define COLOR_GRAY_CC   0x00cccccc
 #define COLOR_GRAY_FF   0x00ffffff
 
-#define GTH_IMAGE_VIEWER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTH_TYPE_IMAGE_VIEWER, GthImageViewerPrivate))
 
 #define DRAG_THRESHOLD  1     /* When dragging the image ignores movements
 			       * smaller than this value. */
@@ -251,16 +250,18 @@ get_zoomed_size (GthImageViewer *viewer,
 static void
 _gth_image_viewer_update_image_area (GthImageViewer *viewer)
 {
-	GtkWidget *widget;
-	int        pixbuf_width;
-	int        pixbuf_height;
-	int        gdk_width;
-	int        gdk_height;
+	GtkWidget     *widget;
+	int            pixbuf_width;
+	int            pixbuf_height;
+	GtkAllocation  allocation;
+	int            gdk_width;
+	int            gdk_height;
 
 	widget = GTK_WIDGET (viewer);
 	get_zoomed_size (viewer, &pixbuf_width, &pixbuf_height, viewer->priv->zoom_level);
-	gdk_width = widget->allocation.width - viewer->priv->frame_border2;
-	gdk_height = widget->allocation.height - viewer->priv->frame_border2;
+	gtk_widget_get_allocation (widget, &allocation);
+	gdk_width = allocation.width - viewer->priv->frame_border2;
+	gdk_height = allocation.height - viewer->priv->frame_border2;
 
 	viewer->image_area.x = MAX (viewer->priv->frame_border, (gdk_width - pixbuf_width) / 2);
 	viewer->image_area.y = MAX (viewer->priv->frame_border, (gdk_height - pixbuf_height) / 2);
@@ -275,15 +276,17 @@ set_zoom (GthImageViewer *viewer,
 	  int             center_x,
 	  int             center_y)
 {
-	GtkWidget *widget = (GtkWidget*) viewer;
-	gdouble    zoom_ratio;
-	int        gdk_width, gdk_height;
+	GtkWidget     *widget = (GtkWidget*) viewer;
+	GtkAllocation  allocation;
+	gdouble        zoom_ratio;
+	int            gdk_width, gdk_height;
 
 	g_return_if_fail (viewer != NULL);
 	g_return_if_fail (viewer->priv->loader != NULL);
 
-	gdk_width = widget->allocation.width - viewer->priv->frame_border2;
-	gdk_height = widget->allocation.height - viewer->priv->frame_border2;
+	gtk_widget_get_allocation (widget, &allocation);
+	gdk_width = allocation.width - viewer->priv->frame_border2;
+	gdk_height = allocation.height - viewer->priv->frame_border2;
 
 	/* try to keep the center of the view visible. */
 
@@ -326,19 +329,23 @@ static void
 gth_image_viewer_realize (GtkWidget *widget)
 {
 	GthImageViewer *viewer;
+	GtkAllocation   allocation;
 	GdkWindowAttr   attributes;
 	int             attributes_mask;
+	GdkWindow      *window;
+	GtkStyle       *style;
 
 	g_return_if_fail (GTH_IS_IMAGE_VIEWER (widget));
 
 	viewer = GTH_IMAGE_VIEWER (widget);
-	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+	gtk_widget_set_realized (widget, TRUE);
+	gtk_widget_get_allocation (widget, &allocation);
 
 	attributes.window_type = GDK_WINDOW_CHILD;
-	attributes.x           = widget->allocation.x;
-	attributes.y           = widget->allocation.y;
-	attributes.width       = widget->allocation.width;
-	attributes.height      = widget->allocation.height;
+	attributes.x           = allocation.x;
+	attributes.y           = allocation.y;
+	attributes.width       = allocation.width;
+	attributes.height      = allocation.height;
 	attributes.wclass      = GDK_INPUT_OUTPUT;
 	attributes.visual      = gtk_widget_get_visual (widget);
 	attributes.colormap    = gtk_widget_get_colormap (widget);
@@ -355,23 +362,27 @@ gth_image_viewer_realize (GtkWidget *widget)
 				  | GDK_WA_Y
 				  | GDK_WA_VISUAL
 				  | GDK_WA_COLORMAP);
-	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-					 &attributes,
-					 attributes_mask);
-	gdk_window_set_user_data (widget->window, viewer);
 
-	widget->style = gtk_style_attach (widget->style, widget->window);
-	gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+	window = gdk_window_new (gtk_widget_get_parent_window (widget),
+				 &attributes,
+				 attributes_mask);
+	gtk_widget_set_window (widget, window);
+	gdk_window_set_user_data (window, viewer);
+
+	style = gtk_widget_get_style (widget);
+	style = gtk_style_attach (style, window);
+	gtk_style_set_background (style, window, GTK_STATE_NORMAL);
+	gtk_widget_set_style (widget, style);
 
 	viewer->priv->cursor = gdk_cursor_new (GDK_LEFT_PTR);
-	viewer->priv->cursor_void = gth_cursor_get (widget->window, GTH_CURSOR_VOID);
-	gdk_window_set_cursor (widget->window, viewer->priv->cursor);
+	viewer->priv->cursor_void = gth_cursor_get (window, GTH_CURSOR_VOID);
+	gdk_window_set_cursor (window, viewer->priv->cursor);
 
 	if (viewer->priv->transp_type == GTH_TRANSP_TYPE_NONE) {
 		GdkColor color;
 		guint    base_color;
 
-		color = GTK_WIDGET (viewer)->style->bg[GTK_STATE_NORMAL];
+		color = style->bg[GTK_STATE_NORMAL];
 		base_color = (0xFF000000
 			      | (to_255 (color.red) << 16)
 			      | (to_255 (color.green) << 8)
@@ -440,17 +451,21 @@ gth_image_viewer_unmap (GtkWidget *widget)
 static void
 zoom_to_fit (GthImageViewer *viewer)
 {
-	GdkPixbuf *buf;
-	int        gdk_width, gdk_height;
-	double     x_level, y_level;
-	double     new_zoom_level;
+	GdkPixbuf     *pixbuf;
+	GtkAllocation  allocation;
+	int            gdk_width;
+	int            gdk_height;
+	double         x_level;
+	double         y_level;
+	double         new_zoom_level;
 
-	buf = gth_image_viewer_get_current_pixbuf (viewer);
+	pixbuf = gth_image_viewer_get_current_pixbuf (viewer);
 
-	gdk_width = GTK_WIDGET (viewer)->allocation.width - viewer->priv->frame_border2;
-	gdk_height = GTK_WIDGET (viewer)->allocation.height - viewer->priv->frame_border2;
-	x_level = (double) gdk_width / gdk_pixbuf_get_width (buf);
-	y_level = (double) gdk_height / gdk_pixbuf_get_height (buf);
+	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
+	gdk_width = allocation.width - viewer->priv->frame_border2;
+	gdk_height = allocation.height - viewer->priv->frame_border2;
+	x_level = (double) gdk_width / gdk_pixbuf_get_width (pixbuf);
+	y_level = (double) gdk_height / gdk_pixbuf_get_height (pixbuf);
 
 	new_zoom_level = (x_level < y_level) ? x_level : y_level;
 	if (new_zoom_level > 0.0) {
@@ -464,14 +479,15 @@ zoom_to_fit (GthImageViewer *viewer)
 static void
 zoom_to_fit_width (GthImageViewer *viewer)
 {
-	GdkPixbuf *buf;
-	double     new_zoom_level;
-	int        gdk_width;
+	GdkPixbuf     *pixbuf;
+	GtkAllocation  allocation;
+	int            gdk_width;
+	double         new_zoom_level;
 
-	buf = gth_image_viewer_get_current_pixbuf (viewer);
-
-	gdk_width = GTK_WIDGET (viewer)->allocation.width - viewer->priv->frame_border2;
-	new_zoom_level = (double) gdk_width / gdk_pixbuf_get_width (buf);
+	pixbuf = gth_image_viewer_get_current_pixbuf (viewer);
+	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
+	gdk_width = allocation.width - viewer->priv->frame_border2;
+	new_zoom_level = (double) gdk_width / gdk_pixbuf_get_width (pixbuf);
 
 	if (new_zoom_level > 0.0) {
 		viewer->priv->doing_zoom_fit = TRUE;
@@ -491,10 +507,14 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 
 	viewer = GTH_IMAGE_VIEWER (widget);
 
-	widget->allocation = *allocation;
+	gtk_widget_set_allocation (widget, allocation);
+
 	gdk_width = allocation->width - viewer->priv->frame_border2;
 	gdk_height = allocation->height - viewer->priv->frame_border2;
 
+	if ((gdk_width < 0) || (gdk_height < 0))
+		return;
+
 	current_pixbuf = gth_image_viewer_get_current_pixbuf (viewer);
 
 	/* If a fit type is active update the zoom level. */
@@ -504,6 +524,7 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 		case GTH_FIT_SIZE:
 			zoom_to_fit (viewer);
 			break;
+
 		case GTH_FIT_SIZE_IF_LARGER:
 			if ((gdk_width < gdk_pixbuf_get_width (current_pixbuf))
 				|| (gdk_height < gdk_pixbuf_get_height (current_pixbuf)))
@@ -516,9 +537,11 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 				viewer->priv->doing_zoom_fit = FALSE;
 			}
 			break;
+
 		case GTH_FIT_WIDTH:
 			zoom_to_fit_width (viewer);
 			break;
+
 		case GTH_FIT_WIDTH_IF_LARGER:
 			if (gdk_width < gdk_pixbuf_get_width (current_pixbuf)) {
 				zoom_to_fit_width (viewer);
@@ -529,6 +552,7 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 				viewer->priv->doing_zoom_fit = FALSE;
 			}
 			break;
+
 		default:
 			break;
 		}
@@ -537,7 +561,8 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 	/* Check whether the offset is still valid. */
 
 	if (current_pixbuf != NULL) {
-		int width, height;
+		int width;
+		int height;
 
 		get_zoomed_size (viewer, &width, &height, viewer->priv->zoom_level);
 
@@ -555,37 +580,37 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 		else
 			viewer->y_offset = 0;
 
-		if ((width != viewer->hadj->upper) || (height != viewer->vadj->upper))
+		if ((width != gtk_adjustment_get_upper (viewer->hadj)) || (height != gtk_adjustment_get_upper (viewer->vadj)))
 			g_signal_emit (G_OBJECT (viewer),
 				       gth_image_viewer_signals[SIZE_CHANGED],
 				       0);
 
 		/* Change adjustment values. */
 
-		viewer->hadj->lower          = 0.0;
-		viewer->hadj->upper          = width;
-		viewer->hadj->value          = viewer->x_offset;
-		viewer->hadj->step_increment = STEP_INCREMENT;
-		viewer->hadj->page_increment = gdk_width / 2;
-		viewer->hadj->page_size      = gdk_width;
+		gtk_adjustment_set_lower (viewer->hadj, 0.0);
+		gtk_adjustment_set_upper (viewer->hadj, width);
+		gtk_adjustment_set_value (viewer->hadj, viewer->x_offset);
+		gtk_adjustment_set_step_increment (viewer->hadj, STEP_INCREMENT);
+		gtk_adjustment_set_page_increment (viewer->hadj, gdk_width / 2);
+		gtk_adjustment_set_page_size (viewer->hadj, gdk_width);
 
-		viewer->vadj->lower          = 0.0;
-		viewer->vadj->upper          = height;
-		viewer->vadj->value          = viewer->y_offset;
-		viewer->vadj->step_increment = STEP_INCREMENT;
-		viewer->vadj->page_increment = gdk_height / 2;
-		viewer->vadj->page_size      = gdk_height;
+		gtk_adjustment_set_lower (viewer->vadj, 0.0);
+		gtk_adjustment_set_upper (viewer->vadj, height);
+		gtk_adjustment_set_value (viewer->vadj, viewer->y_offset);
+		gtk_adjustment_set_step_increment (viewer->vadj, STEP_INCREMENT);
+		gtk_adjustment_set_page_increment (viewer->vadj, gdk_height / 2);
+		gtk_adjustment_set_page_size (viewer->vadj, gdk_height);
 	}
 	else {
-		viewer->hadj->lower     = 0.0;
-		viewer->hadj->upper     = 1.0;
-		viewer->hadj->value     = 0.0;
-		viewer->hadj->page_size = 1.0;
+		gtk_adjustment_set_lower (viewer->hadj, 0.0);
+		gtk_adjustment_set_upper (viewer->hadj, 1.0);
+		gtk_adjustment_set_value (viewer->hadj, 0.0);
+		gtk_adjustment_set_page_size (viewer->hadj, 1.0);
 
-		viewer->vadj->lower     = 0.0;
-		viewer->vadj->upper     = 1.0;
-		viewer->vadj->value     = 0.0;
-		viewer->vadj->page_size = 1.0;
+		gtk_adjustment_set_lower (viewer->vadj, 0.0);
+		gtk_adjustment_set_upper (viewer->vadj, 1.0);
+		gtk_adjustment_set_value (viewer->vadj, 0.0);
+		gtk_adjustment_set_page_size (viewer->vadj, 1.0);
 	}
 
 	_gth_image_viewer_update_image_area (viewer);
@@ -599,10 +624,12 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 
 	/**/
 
-	if (GTK_WIDGET_REALIZED (widget))
-		gdk_window_move_resize (widget->window,
-					allocation->x, allocation->y,
-					allocation->width, allocation->height);
+	if (gtk_widget_get_realized (widget))
+		gdk_window_move_resize (gtk_widget_get_window (widget),
+					allocation->x,
+					allocation->y,
+					allocation->width,
+					allocation->height);
 
 	gth_image_viewer_tool_size_allocate (viewer->priv->tool, allocation);
 
@@ -619,9 +646,7 @@ static gboolean
 gth_image_viewer_focus_in (GtkWidget     *widget,
 			   GdkEventFocus *event)
 {
-	GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
 	gtk_widget_queue_draw (widget);
-
 	return TRUE;
 }
 
@@ -630,9 +655,7 @@ static gboolean
 gth_image_viewer_focus_out (GtkWidget     *widget,
 			    GdkEventFocus *event)
 {
-	GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
 	gtk_widget_queue_draw (widget);
-
 	return TRUE;
 }
 
@@ -712,89 +735,91 @@ gth_image_viewer_expose (GtkWidget      *widget,
 			 GdkEventExpose *event)
 {
 	GthImageViewer *viewer;
+	cairo_t        *cr;
+	GtkAllocation   allocation;
 	int             gdk_width;
 	int             gdk_height;
+	GtkStyle       *style;
 
 	viewer = GTH_IMAGE_VIEWER (widget);
 
+	cr = gdk_cairo_create (gtk_widget_get_window (widget));
+
+	cairo_set_line_width (cr, 0.5);
+	cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
+
+	gdk_cairo_region (cr, event->region);
+	cairo_clip (cr);
+
 	/* Draw the background. */
 
-	gdk_width = widget->allocation.width - viewer->priv->frame_border2;
-	gdk_height = widget->allocation.height - viewer->priv->frame_border2;
+	gtk_widget_get_allocation (widget, &allocation);
+	gdk_width = allocation.width - viewer->priv->frame_border2;
+	gdk_height = allocation.height - viewer->priv->frame_border2;
+
+	style = gtk_widget_get_style (widget);
 
 	if ((viewer->image_area.x > viewer->priv->frame_border)
 	    || (viewer->image_area.y > viewer->priv->frame_border)
 	    || (viewer->image_area.width < gdk_width)
 	    || (viewer->image_area.height < gdk_height))
 	{
-		int    rx, ry, rw, rh;
-		GdkGC *gc;
+		int rx, ry, rw, rh;
 
 		if (viewer->priv->black_bg)
-			gc = widget->style->black_gc;
+			cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 		else
-			gc = widget->style->bg_gc[GTK_STATE_NORMAL];
+			gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_NORMAL]);
 
 		if (gth_image_viewer_get_current_pixbuf (viewer) == NULL) {
-			gdk_draw_rectangle (widget->window,
-					    gc,
-					    TRUE,
-					    0, 0,
-					    widget->allocation.width,
-					    widget->allocation.height);
+			cairo_rectangle (cr,
+					 0,
+					 0,
+					 allocation.width,
+					 allocation.height);
 		}
 		else {
 			/* If an image is present draw in four phases to avoid
 			 * flickering. */
 
 			/* Top rectangle. */
+
 			rx = 0;
 			ry = 0;
-			rw = widget->allocation.width;
+			rw = allocation.width;
 			rh = viewer->image_area.y;
 			if ((rw > 0) && (rh > 0))
-				gdk_draw_rectangle (widget->window,
-						    gc,
-						    TRUE,
-						    rx, ry,
-						    rw, rh);
+				cairo_rectangle (cr, rx, ry, rw, rh);
 
 			/* Bottom rectangle. */
+
 			rx = 0;
 			ry = viewer->image_area.y + viewer->image_area.height;
-			rw = widget->allocation.width;
-			rh = widget->allocation.height - viewer->image_area.y - viewer->image_area.height;
+			rw = allocation.width;
+			rh = allocation.height - viewer->image_area.y - viewer->image_area.height;
 			if ((rw > 0) && (rh > 0))
-				gdk_draw_rectangle (widget->window,
-						    gc,
-						    TRUE,
-						    rx, ry,
-						    rw, rh);
+				cairo_rectangle (cr, rx, ry, rw, rh);
 
 			/* Left rectangle. */
+
 			rx = 0;
 			ry = viewer->image_area.y - 1;
 			rw = viewer->image_area.x;
 			rh = viewer->image_area.height + 2;
 			if ((rw > 0) && (rh > 0))
-				gdk_draw_rectangle (widget->window,
-						    gc,
-						    TRUE,
-						    rx, ry,
-						    rw, rh);
+				cairo_rectangle (cr, rx, ry, rw, rh);
 
 			/* Right rectangle. */
+
 			rx = viewer->image_area.x + viewer->image_area.width;
 			ry = viewer->image_area.y - 1;
-			rw = widget->allocation.width - viewer->image_area.x - viewer->image_area.width;
+			rw = allocation.width - viewer->image_area.x - viewer->image_area.width;
 			rh = viewer->image_area.height + 2;
 			if ((rw > 0) && (rh > 0))
-				gdk_draw_rectangle (widget->window,
-						    gc,
-						    TRUE,
-						    rx, ry,
-						    rw, rh);
+				cairo_rectangle (cr, rx, ry, rw, rh);
 		}
+
+		cairo_fill (cr);
 	}
 
 	/* Draw the frame. */
@@ -802,58 +827,43 @@ gth_image_viewer_expose (GtkWidget      *widget,
 	if ((viewer->priv->frame_border > 0)
 	    && (gth_image_viewer_get_current_pixbuf (viewer) != NULL))
 	{
-		int    x1, y1, x2, y2;
-		GdkGC *gc;
+
+		/* bottom and right side */
 
 		if (viewer->priv->black_bg)
-			gc = widget->style->black_gc;
+			cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 		else
-			gc = widget->style->light_gc[GTK_STATE_NORMAL];
-			/*gc = widget->style->dark_gc[GTK_STATE_NORMAL];*/
+			gdk_cairo_set_source_color (cr, &style->light[GTK_STATE_NORMAL]);
 
-		x1 = viewer->image_area.x + viewer->image_area.width;
-		y1 = viewer->image_area.y - 1;
-		x2 = viewer->image_area.x + viewer->image_area.width;
-		y2 = viewer->image_area.y + viewer->image_area.height;
-		gdk_draw_line (widget->window,
-			       gc,
-			       x1, y1,
-			       x2, y2);
+		cairo_move_to (cr,
+			       viewer->image_area.x + viewer->image_area.width + 0.5,
+			       viewer->image_area.y - 1 + 0.5);
+		cairo_line_to (cr,
+			       viewer->image_area.x + viewer->image_area.width + 0.5,
+			       viewer->image_area.y + viewer->image_area.height + 0.5);
+		cairo_line_to (cr,
+			       viewer->image_area.x - 1 + 0.5,
+			       viewer->image_area.y + viewer->image_area.height + 0.5);
+		cairo_stroke (cr);
 
-		x1 = viewer->image_area.x - 1;
-		y1 = viewer->image_area.y + viewer->image_area.height;
-		x2 = viewer->image_area.x + viewer->image_area.width;
-		y2 = viewer->image_area.y + viewer->image_area.height;
-		gdk_draw_line (widget->window,
-			       gc,
-			       x1, y1,
-			       x2, y2);
+		/* top and left side */
 
-		if (viewer->priv->black_bg)
-			gc = widget->style->black_gc;
-		else
-			gc = widget->style->dark_gc[GTK_STATE_NORMAL];
+		if (! viewer->priv->black_bg)
+			gdk_cairo_set_source_color (cr, &style->dark[GTK_STATE_NORMAL]);
 
-		x1 = viewer->image_area.x - 1;
-		y1 = viewer->image_area.y - 1;
-		x2 = viewer->image_area.x - 1;
-		y2 = viewer->image_area.y + viewer->image_area.height;
-		gdk_draw_line (widget->window,
-			       gc,
-			       x1, y1,
-			       x2, y2);
-
-		x1 = viewer->image_area.x - 1;
-		y1 = viewer->image_area.y - 1;
-		x2 = viewer->image_area.x + viewer->image_area.width;
-		y2 = viewer->image_area.y - 1;
-		gdk_draw_line (widget->window,
-			       gc,
-			       x1, y1,
-			       x2, y2);
+		cairo_move_to (cr,
+			       viewer->image_area.x - 1 + 0.5,
+			       viewer->image_area.y + viewer->image_area.height + 0.5);
+		cairo_line_to (cr,
+			       viewer->image_area.x - 1 + 0.5,
+			       viewer->image_area.y - 1 + 0.5);
+		cairo_line_to (cr,
+			       viewer->image_area.x + viewer->image_area.width + 0.5,
+			       viewer->image_area.y - 1 + 0.5);
+		cairo_stroke (cr);
 	}
 
-	gth_image_viewer_tool_expose (viewer->priv->tool, &event->area);
+	gth_image_viewer_tool_expose (viewer->priv->tool, event, cr);
 
 	/* Draw the focus. */
 
@@ -877,6 +887,8 @@ gth_image_viewer_expose (GtkWidget      *widget,
 
 	queue_frame_change (viewer);
 
+	cairo_destroy (cr);
+
 	return FALSE;
 }
 
@@ -888,7 +900,7 @@ gth_image_viewer_button_press (GtkWidget      *widget,
 	GthImageViewer *viewer = GTH_IMAGE_VIEWER (widget);
 	int             retval;
 
-	if (! GTK_WIDGET_HAS_FOCUS (widget)) {
+	if (! gtk_widget_has_focus (widget)) {
 		gtk_widget_grab_focus (widget);
 		viewer->priv->just_focused = TRUE;
 	}
@@ -960,8 +972,11 @@ expose_area (GthImageViewer *viewer,
 	event.area.y = y;
 	event.area.width = width;
 	event.area.height = height;
+	event.region = gdk_region_rectangle (&event.area);
 
 	gth_image_viewer_expose (GTK_WIDGET (viewer), &event);
+
+	gdk_region_destroy (event.region);
 }
 
 
@@ -970,10 +985,11 @@ scroll_to (GthImageViewer *viewer,
 	   int            *x_offset,
 	   int            *y_offset)
 {
-	GdkDrawable *drawable;
-	int          width, height;
-	int          delta_x, delta_y;
-	int          gdk_width, gdk_height;
+	GdkDrawable   *drawable;
+	GtkAllocation  allocation;
+	int            width, height;
+	int            delta_x, delta_y;
+	int            gdk_width, gdk_height;
 
 	g_return_if_fail (viewer != NULL);
 
@@ -982,9 +998,10 @@ scroll_to (GthImageViewer *viewer,
 
 	get_zoomed_size (viewer, &width, &height, viewer->priv->zoom_level);
 
-	drawable = GTK_WIDGET (viewer)->window;
-	gdk_width = GTK_WIDGET (viewer)->allocation.width - viewer->priv->frame_border2;
-	gdk_height = GTK_WIDGET (viewer)->allocation.height - viewer->priv->frame_border2;
+	drawable = gtk_widget_get_window (GTK_WIDGET (viewer));
+	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
+	gdk_width = allocation.width - viewer->priv->frame_border2;
+	gdk_height = allocation.height - viewer->priv->frame_border2;
 
 	if (width > gdk_width)
 		*x_offset = CLAMP (*x_offset, 0, width - gdk_width);
@@ -1012,17 +1029,15 @@ scroll_to (GthImageViewer *viewer,
 			       gth_image_viewer_signals[REPAINTED],
 			       0);
 
-		expose_area (viewer, 0, 0,
-			     GTK_WIDGET (viewer)->allocation.width,
-			     GTK_WIDGET (viewer)->allocation.height);
+		expose_area (viewer, 0, 0, allocation.width, allocation.height);
 
 		return;
 	}
 
 	if ((delta_x != 0) || (delta_y != 0)) {
-		GdkGC *gc = GTK_WIDGET (viewer)->style->black_gc;
-		int    src_x, dest_x;
-		int    src_y, dest_y;
+		int      src_x, dest_x;
+		int      src_y, dest_y;
+		cairo_t *cr;
 
 		if (delta_x < 0) {
 			src_x = 0;
@@ -1042,23 +1057,21 @@ scroll_to (GthImageViewer *viewer,
 			dest_y = 0;
 		}
 
-		gc = gdk_gc_new (drawable);
-		gdk_gc_set_exposures (gc, TRUE);
-
 		dest_x += viewer->priv->frame_border;
 		dest_y += viewer->priv->frame_border;
 		src_x += viewer->priv->frame_border;
 		src_y += viewer->priv->frame_border;
 
-		gdk_draw_drawable (drawable,
-				   gc,
-				   drawable,
-				   src_x, src_y,
-				   dest_x, dest_y,
-				   gdk_width - abs (delta_x),
-				   gdk_height - abs (delta_y));
+		cr = gdk_cairo_create (drawable);
+		gdk_cairo_set_source_pixmap (cr, drawable, dest_x - src_x, dest_y - src_y);
+		cairo_rectangle (cr,
+				 dest_x,
+				 dest_y,
+				 gdk_width - abs (delta_x),
+				 gdk_height - abs (delta_y));
+		cairo_fill (cr);
 
-		g_object_unref (gc);
+		cairo_destroy (cr);
 	}
 
 	viewer->x_offset = *x_offset;
@@ -1134,13 +1147,11 @@ gth_image_viewer_scroll_event (GtkWidget      *widget,
 		return FALSE;
 
 	adj = viewer->hadj;
-
 	if (event->direction == GDK_SCROLL_LEFT)
-		new_value = adj->value - adj->page_increment / 2;
+		new_value = gtk_adjustment_get_value (adj) - gtk_adjustment_get_page_increment (adj) / 2;
 	else if (event->direction == GDK_SCROLL_RIGHT)
-		new_value = adj->value + adj->page_increment / 2;
-
-	new_value = CLAMP (new_value, adj->lower, adj->upper - adj->page_size);
+		new_value = gtk_adjustment_get_value (adj) + gtk_adjustment_get_page_increment (adj) / 2;
+	new_value = CLAMP (new_value, gtk_adjustment_get_lower (adj), gtk_adjustment_get_upper (adj) - gtk_adjustment_get_page_size (adj));
 	gtk_adjustment_set_value (adj, new_value);
 
 	return TRUE;
@@ -1156,10 +1167,12 @@ gth_image_viewer_style_set (GtkWidget *widget,
 	GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
 
 	if (viewer->priv->transp_type == GTH_TRANSP_TYPE_NONE) {
-		GdkColor color;
-		guint    base_color;
+		GtkStyle *style;
+		GdkColor  color;
+		guint     base_color;
 
-		color = GTK_WIDGET (viewer)->style->bg[GTK_STATE_NORMAL];
+		style = gtk_widget_get_style (GTK_WIDGET (viewer));
+		color = style->bg[GTK_STATE_NORMAL];
 		base_color = (0xFF000000
 			      | (to_255 (color.red) << 16)
 			      | (to_255 (color.green) << 8)
@@ -1192,16 +1205,16 @@ scroll_signal (GtkWidget     *widget,
 
 	switch (xscroll_type) {
 	case GTK_SCROLL_STEP_LEFT:
-		xstep = -viewer->hadj->step_increment;
+		xstep = - gtk_adjustment_get_step_increment (viewer->hadj);
 		break;
 	case GTK_SCROLL_STEP_RIGHT:
-		xstep = viewer->hadj->step_increment;
+		xstep = gtk_adjustment_get_step_increment (viewer->hadj);
 		break;
 	case GTK_SCROLL_PAGE_LEFT:
-		xstep = -viewer->hadj->page_increment;
+		xstep = - gtk_adjustment_get_page_increment (viewer->hadj);
 		break;
 	case GTK_SCROLL_PAGE_RIGHT:
-		xstep = viewer->hadj->page_increment;
+		xstep = gtk_adjustment_get_page_increment (viewer->hadj);
 		break;
 	default:
 		xstep = 0;
@@ -1210,16 +1223,16 @@ scroll_signal (GtkWidget     *widget,
 
 	switch (yscroll_type) {
 	case GTK_SCROLL_STEP_UP:
-		ystep = -viewer->vadj->step_increment;
+		ystep = - gtk_adjustment_get_step_increment (viewer->vadj);
 		break;
 	case GTK_SCROLL_STEP_DOWN:
-		ystep = viewer->vadj->step_increment;
+		ystep = gtk_adjustment_get_step_increment (viewer->vadj);
 		break;
 	case GTK_SCROLL_PAGE_UP:
-		ystep = -viewer->vadj->page_increment;
+		ystep = - gtk_adjustment_get_page_increment (viewer->vadj);
 		break;
 	case GTK_SCROLL_PAGE_DOWN:
-		ystep = viewer->vadj->page_increment;
+		ystep = gtk_adjustment_get_page_increment (viewer->vadj);
 		break;
 	default:
 		ystep = 0;
@@ -1231,12 +1244,12 @@ scroll_signal (GtkWidget     *widget,
 
 
 static gboolean
-hadj_value_changed (GtkObject      *adj,
+hadj_value_changed (GtkAdjustment  *adj,
 		    GthImageViewer *viewer)
 {
 	int x_ofs, y_ofs;
 
-	x_ofs = (int) GTK_ADJUSTMENT (adj)->value;
+	x_ofs = (int) gtk_adjustment_get_value (adj);
 	y_ofs = viewer->y_offset;
 	scroll_to (viewer, &x_ofs, &y_ofs);
 
@@ -1245,13 +1258,13 @@ hadj_value_changed (GtkObject      *adj,
 
 
 static gboolean
-vadj_value_changed (GtkObject      *adj,
+vadj_value_changed (GtkAdjustment  *adj,
 		    GthImageViewer *viewer)
 {
 	int x_ofs, y_ofs;
 
 	x_ofs = viewer->x_offset;
-	y_ofs = (int) GTK_ADJUSTMENT (adj)->value;
+	y_ofs = (int) gtk_adjustment_get_value (adj);
 	scroll_to (viewer, &x_ofs, &y_ofs);
 
 	return FALSE;
@@ -1633,7 +1646,6 @@ image_loader_ready_cb (GthImageLoader *il,
 
 	if (error != NULL) {
 		g_clear_error (&error);
-
 		gth_image_viewer_set_void (viewer);
 		g_signal_emit (G_OBJECT (viewer),
 			       gth_image_viewer_signals[IMAGE_READY],
@@ -1702,12 +1714,13 @@ image_loader_ready_cb (GthImageLoader *il,
 static void
 gth_image_viewer_instance_init (GthImageViewer *viewer)
 {
-	GTK_WIDGET_SET_FLAGS (viewer, GTK_CAN_FOCUS);
-
-	viewer->priv = GTH_IMAGE_VIEWER_GET_PRIVATE (viewer);
+	gtk_widget_set_can_focus (GTK_WIDGET (viewer), TRUE);
+	gtk_widget_set_double_buffered (GTK_WIDGET (viewer), TRUE);
 
 	/* Initialize data. */
 
+	viewer->priv = G_TYPE_INSTANCE_GET_PRIVATE (viewer, GTH_TYPE_IMAGE_VIEWER, GthImageViewerPrivate);
+
 	viewer->priv->check_type = GTH_CHECK_TYPE_MIDTONE;
 	viewer->priv->check_size = GTH_CHECK_SIZE_LARGE;
 	viewer->priv->check_color1 = COLOR_GRAY_66;
@@ -2171,12 +2184,13 @@ void
 gth_image_viewer_set_zoom (GthImageViewer *viewer,
 			   gdouble         zoom_level)
 {
-	GtkWidget *widget = (GtkWidget*) viewer;
+	GtkAllocation allocation;
 
+	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
 	set_zoom (viewer,
 		  zoom_level,
-		  (widget->allocation.width - viewer->priv->frame_border2) / 2,
-		  (widget->allocation.height - viewer->priv->frame_border2) / 2);
+		  (allocation.width - viewer->priv->frame_border2) / 2,
+		  (allocation.height - viewer->priv->frame_border2) / 2);
 }
 
 
@@ -2257,14 +2271,16 @@ void
 gth_image_viewer_set_transp_type (GthImageViewer *viewer,
 				  GthTranspType   transp_type)
 {
-	GdkColor color;
-	guint    base_color;
+	GtkStyle *style;
+	GdkColor  color;
+	guint     base_color;
 
 	g_return_if_fail (viewer != NULL);
 
 	viewer->priv->transp_type = transp_type;
 
-	color = GTK_WIDGET (viewer)->style->bg[GTK_STATE_NORMAL];
+	style = gtk_widget_get_style (GTK_WIDGET (viewer));
+	color = style->bg[GTK_STATE_NORMAL];
 	base_color = (0xFF000000
 		      | (to_255 (color.red) << 16)
 		      | (to_255 (color.green) << 8)
@@ -2357,8 +2373,12 @@ gth_image_viewer_set_size_request (GthImageViewer *viewer,
 				   int             width,
 				   int             height)
 {
-	GTK_WIDGET (viewer)->requisition.width = width;
-	GTK_WIDGET (viewer)->requisition.height = height;
+	GtkRequisition requisition;
+
+	requisition.width = width;
+	requisition.height = height;
+	gtk_widget_size_request (GTK_WIDGET (viewer), &requisition);
+
 	gtk_widget_queue_resize (GTK_WIDGET (viewer));
 }
 
@@ -2403,7 +2423,7 @@ gth_image_viewer_set_tool (GthImageViewer     *viewer,
 		viewer->priv->tool = gth_image_dragger_new (viewer);
 	else
 		viewer->priv->tool = g_object_ref (tool);
-	if (GTK_WIDGET_REALIZED (viewer))
+	if (gtk_widget_get_realized (GTK_WIDGET (viewer)))
 		gth_image_viewer_tool_realize (viewer->priv->tool);
 	gth_image_viewer_tool_image_changed (viewer->priv->tool);
 	gtk_widget_queue_resize (GTK_WIDGET (viewer));
@@ -2437,7 +2457,7 @@ gth_image_viewer_scroll_step_x (GthImageViewer *viewer,
 				gboolean        increment)
 {
 	scroll_relative (viewer,
-			 (increment ? 1 : -1) * viewer->hadj->step_increment,
+			 (increment ? 1 : -1) * gtk_adjustment_get_step_increment (viewer->hadj),
 			 0);
 }
 
@@ -2448,7 +2468,7 @@ gth_image_viewer_scroll_step_y (GthImageViewer *viewer,
 {
 	scroll_relative (viewer,
 			 0,
-			 (increment ? 1 : -1) * viewer->vadj->step_increment);
+			 (increment ? 1 : -1) * gtk_adjustment_get_step_increment (viewer->vadj));
 }
 
 
@@ -2457,7 +2477,7 @@ gth_image_viewer_scroll_page_x (GthImageViewer *viewer,
 				gboolean        increment)
 {
 	scroll_relative (viewer,
-			 (increment ? 1 : -1) * viewer->hadj->page_increment,
+			 (increment ? 1 : -1) * gtk_adjustment_get_page_increment (viewer->hadj),
 			 0);
 }
 
@@ -2468,7 +2488,7 @@ gth_image_viewer_scroll_page_y (GthImageViewer *viewer,
 {
 	scroll_relative (viewer,
 			 0,
-			 (increment ? 1 : -1) * viewer->vadj->page_increment);
+			 (increment ? 1 : -1) * gtk_adjustment_get_page_increment (viewer->vadj));
 }
 
 
@@ -2504,7 +2524,7 @@ gth_image_viewer_show_cursor (GthImageViewer *viewer)
 		return;
 
 	viewer->priv->cursor_visible = TRUE;
-	gdk_window_set_cursor (GTK_WIDGET (viewer)->window, viewer->priv->cursor);
+	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (viewer)), viewer->priv->cursor);
 }
 
 
@@ -2515,7 +2535,7 @@ gth_image_viewer_hide_cursor (GthImageViewer *viewer)
 		return;
 
 	viewer->priv->cursor_visible = FALSE;
-	gdk_window_set_cursor (GTK_WIDGET (viewer)->window, viewer->priv->cursor_void);
+	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (viewer)), viewer->priv->cursor_void);
 }
 
 
@@ -2535,11 +2555,11 @@ gth_image_viewer_set_cursor (GthImageViewer *viewer,
 	else
 		viewer->priv->cursor = gdk_cursor_ref (viewer->priv->cursor_void);
 
-	if (! GTK_WIDGET_REALIZED (viewer))
+	if (! gtk_widget_get_realized (GTK_WIDGET (viewer)))
 		return;
 
 	if (viewer->priv->cursor_visible)
-		gdk_window_set_cursor (GTK_WIDGET (viewer)->window, viewer->priv->cursor);
+		gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (viewer)), viewer->priv->cursor);
 }
 
 
@@ -2581,6 +2601,7 @@ gth_image_viewer_is_frame_visible (GthImageViewer *viewer)
 
 void
 gth_image_viewer_paint (GthImageViewer *viewer,
+			cairo_t        *cr,
 			GdkPixbuf      *pixbuf,
 			int             src_x,
 			int             src_y,
@@ -2590,11 +2611,9 @@ gth_image_viewer_paint (GthImageViewer *viewer,
 			int             height,
 			int             interp_type)
 {
-	double         zoom_level;
-	int            bits_per_sample;
-	GdkColorspace  color_space;
-	guchar        *pixels;
-	int            rowstride;
+	double        zoom_level;
+	int           bits_per_sample;
+	GdkColorspace color_space;
 
 	zoom_level = viewer->priv->zoom_level;
 
@@ -2622,57 +2641,79 @@ gth_image_viewer_paint (GthImageViewer *viewer,
 		viewer->priv->paint_bps = bits_per_sample;
 	}
 
-	if ((zoom_level == 1.0) && ! gdk_pixbuf_get_has_alpha (pixbuf) && (bits_per_sample == 8)) {
-		rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-		pixels = gdk_pixbuf_get_pixels (pixbuf) + (src_y * rowstride) + (src_x * gdk_pixbuf_get_n_channels (pixbuf));
-	}
-	else {
-		if (gdk_pixbuf_get_has_alpha (pixbuf))
-			gdk_pixbuf_composite_color (pixbuf,
-						    viewer->priv->paint_pixbuf,
-						    0, 0,
-						    width, height,
-						    (double) -src_x,
-						    (double) -src_y,
-						    zoom_level,
-						    zoom_level,
-						    interp_type,
-						    255,
-						    src_x, src_y,
-						    viewer->priv->check_size,
-						    viewer->priv->check_color1,
-						    viewer->priv->check_color2);
-		else
-			gdk_pixbuf_scale (pixbuf,
-					  viewer->priv->paint_pixbuf,
-					  0, 0,
-					  width, height,
-					  (double) -src_x,
-					  (double) -src_y,
-					  zoom_level,
-					  zoom_level,
-					  interp_type);
+	if (gdk_pixbuf_get_has_alpha (pixbuf))
+		gdk_pixbuf_composite_color (pixbuf,
+					    viewer->priv->paint_pixbuf,
+					    0, 0,
+					    width, height,
+					    (double) -src_x,
+					    (double) -src_y,
+					    zoom_level,
+					    zoom_level,
+					    interp_type,
+					    255,
+					    src_x, src_y,
+					    viewer->priv->check_size,
+					    viewer->priv->check_color1,
+					    viewer->priv->check_color2);
+	else
+		gdk_pixbuf_scale (pixbuf,
+				  viewer->priv->paint_pixbuf,
+				  0, 0,
+				  width, height,
+				  (double) -src_x,
+				  (double) -src_y,
+				  zoom_level,
+				  zoom_level,
+				  interp_type);
 
-		rowstride = gdk_pixbuf_get_rowstride (viewer->priv->paint_pixbuf);
-		pixels = gdk_pixbuf_get_pixels (viewer->priv->paint_pixbuf);
+	cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
+	gdk_cairo_set_source_pixbuf (cr, viewer->priv->paint_pixbuf, dest_x, dest_y);
+  	cairo_rectangle (cr, dest_x, dest_y, width, height);
+  	cairo_fill (cr);
+}
+
+
+void
+gth_image_viewer_paint_region (GthImageViewer *viewer,
+			       cairo_t        *cr,
+			       GdkPixbuf      *pixbuf,
+			       int             src_x,
+			       int             src_y,
+			       GdkRectangle   *pixbuf_area,
+			       GdkRegion      *region,
+			       int             interp_type)
+{
+	GdkRectangle *rects;
+	int           n_rects;
+	int           i;
+
+	cairo_save (cr);
+	gdk_cairo_region (cr, region);
+	cairo_clip (cr);
+	gdk_cairo_rectangle (cr, pixbuf_area);
+	cairo_clip (cr);
+
+	gdk_region_get_rectangles (region, &rects, &n_rects);
+	for (i = 0; i < n_rects; i++) {
+		GdkRectangle paint_area;
+
+		if (gdk_rectangle_intersect (pixbuf_area, &rects[i], &paint_area))
+			gth_image_viewer_paint (viewer,
+						cr,
+						pixbuf,
+						src_x + paint_area.x,
+						src_y + paint_area.y,
+						paint_area.x,
+						paint_area.y,
+						paint_area.width,
+						paint_area.height,
+						interp_type);
 	}
 
-	gdk_draw_rgb_image_dithalign (GTK_WIDGET (viewer)->window,
-				      GTK_WIDGET (viewer)->style->black_gc,
-				      dest_x, dest_y,
-				      width, height,
-				      GDK_RGB_DITHER_MAX,
-				      pixels,
-				      rowstride,
-				      dest_x, dest_y);
+	cairo_restore (cr);
 
-#if 0
-	gdk_draw_rectangle (GTK_WIDGET (viewer)->window,
-			    GTK_WIDGET (viewer)->style->black_gc,
-			    FALSE,
-			    dest_x, dest_y,
-			    width, height);
-#endif
+	g_free (rects);
 }
 
 
@@ -2680,8 +2721,10 @@ void
 gth_image_viewer_crop_area (GthImageViewer *viewer,
 			    GdkRectangle   *area)
 {
-	GtkWidget *widget = GTK_WIDGET (viewer);
+	GtkWidget     *widget = GTK_WIDGET (viewer);
+	GtkAllocation  allocation;
 
-	area->width = MIN (area->width, widget->allocation.width - viewer->priv->frame_border2);
-	area->width = MIN (area->height, widget->allocation.height - viewer->priv->frame_border2);
+	gtk_widget_get_allocation (widget, &allocation);
+	area->width = MIN (area->width, allocation.width - viewer->priv->frame_border2);
+	area->width = MIN (area->height, allocation.height - viewer->priv->frame_border2);
 }
