@@ -36,6 +36,9 @@ G_BEGIN_DECLS
 #define PREF_STARTUP_LOCATION               "/apps/gthumb/general/startup_location"
 #define PREF_ACTIVE_EXTENSIONS              "/apps/gthumb/general/active_extensions"
 #define PREF_STORE_METADATA_IN_FILES        "/apps/gthumb/general/store_metadata_in_files"
+
+#define PREF_DATA_MIGRATION_CATALOGS_2_10   "/apps/gthumb/data_migration/catalogs_2_10"
+
 #define PREF_GENERAL_FILTER                 "/apps/gthumb/browser/general_filter"
 #define PREF_SHOW_HIDDEN_FILES              "/apps/gthumb/browser/show_hidden_files"
 #define PREF_SHOW_THUMBNAILS                "/apps/gthumb/browser/show_thumbnails"
@@ -69,12 +72,14 @@ G_BEGIN_DECLS
 #define PREF_UI_VIEWER_SIDEBAR_WIDTH        "/apps/gthumb/ui/viewer_sidebar_width"
 #define PREF_UI_PROPERTIES_HEIGHT           "/apps/gthumb/ui/properties_height"
 #define PREF_UI_COMMENT_HEIGHT              "/apps/gthumb/ui/comment_height"
+#define PREF_UI_VIEWER_THUMBNAILS_ORIENT    "/apps/gthumb/ui/viewer_thumbnails_orientation"
 
 #define PREF_ADD_TO_CATALOG_LAST_CATALOG    "/apps/gthumb/dialogs/add_to_catalog/last_catalog"
 #define PREF_ADD_TO_CATALOG_VIEW            "/apps/gthumb/dialogs/add_to_catalog/view"
 
 #define PREF_MSG_CANNOT_MOVE_TO_TRASH       "/apps/gthumb/dialogs/messages/cannot_move_to_trash"
 #define PREF_MSG_SAVE_MODIFIED_IMAGE        "/apps/gthumb/dialogs/messages/save_modified_image"
+#define PREF_MSG_CONFIRM_DELETION           "/apps/gthumb/dialogs/messages/confirm_deletion"
 
 /* default values */
 
@@ -86,7 +91,7 @@ G_BEGIN_DECLS
 #define DEFAULT_THUMBNAIL_SIZE 128
 #define DEFAULT_CONFIRM_DELETION TRUE
 #define DEFAULT_MSG_SAVE_MODIFIED_IMAGE TRUE
-
+#define DEFAULT_MSG_CONFIRM_DELETION TRUE
 
 void             gth_pref_initialize                   (void);
 void             gth_pref_release                      (void);
