@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001-2009 Free Software Foundation, Inc.
+ *  Copyright (C) 2001-2010 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "gth-file-source-vfs.h"
 #include "gth-main.h"
 #include "gth-preferences.h"
+#include "main-migrate.h"
 
 
 enum {
@@ -50,6 +51,9 @@ gboolean NewWindow = FALSE;
 gboolean StartInFullscreen = FALSE;
 gboolean StartSlideshow = FALSE;
 gboolean ImportPhotos = FALSE;
+#ifdef HAVE_CLUTTER
+int      ClutterInitResult = CLUTTER_INIT_ERROR_UNKNOWN;
+#endif
 
 
 static UniqueApp   *gthumb_app;
@@ -87,6 +91,14 @@ static const GOptionEntry options[] = {
 };
 
 
+
+static void
+migrate_data (void)
+{
+	migrate_catalogs_from_2_10 ();
+}
+
+
 static void
 gth_save_state (EggSMClient *client,
 		GKeyFile    *state,
@@ -381,7 +393,7 @@ prepare_application (void)
 int
 main (int argc, char *argv[])
 {
-	GOptionContext *context = NULL;
+	GOptionContext *context;
 	GError         *error = NULL;
 
 	if (! g_thread_supported ())
@@ -403,6 +415,7 @@ main (int argc, char *argv[])
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
 #ifdef HAVE_CLUTTER
+	g_option_context_add_group (context, cogl_get_option_group ());
 	g_option_context_add_group (context, clutter_get_option_group_without_init ());
 #endif
 	if (! g_option_context_parse (context, &argc, &argv, &error)) {
@@ -412,20 +425,20 @@ main (int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-#ifdef HAVE_CLUTTER
-	if (gtk_clutter_init (NULL, NULL) != CLUTTER_INIT_SUCCESS)
-		g_error ("Unable to initialize GtkClutter");
-#endif
-
 	if (version) {
 		g_printf ("%s %s, Copyright (C) 2001-2010 Free Software Foundation, Inc.\n", PACKAGE_NAME, PACKAGE_VERSION);
 		return 0;
 	}
 
+#ifdef HAVE_CLUTTER
+	ClutterInitResult = gtk_clutter_init (NULL, NULL);
+#endif
+
 	/* other initializations */
 
 	gth_session_manager_init ();
 	gth_pref_initialize ();
+	migrate_data ();
 	gth_main_initialize ();
 	prepare_application ();
 
